#! /usr/bin/python2
#-*- coding: utf-8 -*-

from .version import NAME, VERSION

"""-------------------------------------------------------------------------
    Pinguino IDE

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
-------------------------------------------------------------------------"""

import sys
import os
import argparse

from PySide import QtCore, QtGui
from PySide.QtGui import QApplication, QSplashScreen, QPixmap

#----------------------------------------------------------------------
def build_argparse():

    #IDE args
    parsergui = argparse.ArgumentParser(description="*** {PINGUINO_NAME}:GUI ***".format(**os.environ))
    parsergui.add_argument("--lang", dest="lang", nargs=1, default=False, help="set IDE language")
    parsergui.add_argument("--dev", dest="devmode", action="store_true", default=False, help="set IDE developer mode")
    parsergui.add_argument("-style")

    parsed_g, others_g = parsergui.parse_known_args()


    if others_g:
        parsergui.print_help()
        sys.exit()
    else: return parsed_g

parser = build_argparse()

# Python3 compatibility
if os.getenv("PINGUINO_PYTHON") is "3":
    #Python3
    python_path_modules = os.path.join(os.getenv("PINGUINO_LIB"), "batteries", "py3")
else:
    #Python2
    python_path_modules = os.path.join(os.getenv("PINGUINO_LIB"), "batteries", "py2")

if parser.devmode:
    os.environ["PINGUINO_MODE"] = "DEV"
else:
    os.environ["PINGUINO_MODE"] = "NORMAL"

if os.path.isdir(python_path_modules): sys.path.append(python_path_modules)

sys.path.append(os.path.join(os.getenv("PINGUINO_LIB"), "qtgui", "resources"))
# import resources_rc

from .qtgui.ide.ide import PinguinoIDE


def main():

    if parser.lang:
        sys_locale = parser.lang[0]
    else:
        sys_locale = QtCore.QLocale.system().name()
    translator = QtCore.QTranslator()

    #load intern dialogs translations
    qtTranslator = QtCore.QTranslator()
    qtTranslator.load("qt_" + sys_locale, QtCore.QLibraryInfo.location(QtCore.QLibraryInfo.TranslationsPath))

    #load translations files
    translations_path = os.path.join(os.getenv("PINGUINO_LIB"), "multilanguage")
    trasnlations = os.path.exists(translations_path)

    if trasnlations and (os.getenv("PINGUINO_MODE") == "NORMAL"):
        translations_file = "pinguino_" + sys_locale

        if translations_file + ".qm" in os.listdir(translations_path):
            translator.load(os.path.join(os.getenv("PINGUINO_LIB"), "multilanguage", "pinguino_{}.qm".format(sys_locale)))

        elif "_" in sys_locale:
            sys_locale = sys_locale[:sys_locale.find("_")]
            translations_file = "pinguino_" + sys_locale
            if translations_file + ".qm" in os.listdir(translations_path):
                translator.load(os.path.join(os.getenv("PINGUINO_LIB"), "multilanguage", "pinguino_{}.qm".format(sys_locale)))

    app = QApplication(sys.argv)

    #Splash
    pixmap = QPixmap(":/logo/art/splash.png")
    splash = QSplashScreen(pixmap, QtCore.Qt.WindowStaysOnTopHint)

    splash.show()
    splash.setStyleSheet("""
        font-family: inherit;
        font-weight: normal;
        font-size: 11pt;
        """)

    def splash_write(msg):
        if not splash is None:
            splash.showMessage("\t"+msg, color=QtGui.QColor("#4d4d4d"), alignment=QtCore.Qt.AlignBottom)

    splash_write(NAME+" "+VERSION)
    app.processEvents()

    app.installTranslator(qtTranslator)
    if trasnlations:
        app.installTranslator(translator)

    frame = PinguinoIDE(splash_write=splash_write)
    frame.show()

    if not splash is None:
        splash.finish(frame)

    app.exec_()


if __name__ == "__main__":
    main()


#!/usr/bin/env python
#-*- coding: utf-8 -*-

from PySide import QtWebKit
from PySide import QtCore, QtGui
import codecs


########################################################################
class Help(object):
    """"""

    #----------------------------------------------------------------------
    def __init__(self):
        """Constructor"""

        self.connect(self.main.actionPinguino_ide_manual, QtCore.SIGNAL("triggered()"), self.pinguino_ide_manual)
        self.connect(self.main.actionLibraries_help, QtCore.SIGNAL("triggered()"), self.help_libraries)


    #----------------------------------------------------------------------
    def help_libraries(self):
        """"""
        # self.switch_ide_mode(graphical=False)
        self.add_tab("Libraries")
        self.web_view.load("pinguino/docs/index_pinguino/index.html")


    #----------------------------------------------------------------------
    def pinguino_ide_manual(self):
        """"""
        # self.switch_ide_mode(graphical=False)
        self.add_tab("Help")
        self.web_view.load("pinguino/docs/index/index.html")


    #----------------------------------------------------------------------
    def add_tab(self, name):
        tab = QtGui.QWidget()
        setattr(tab, "html", True)
        gridLayout = QtGui.QGridLayout(tab)
        gridLayout.setSpacing(0)
        gridLayout.setContentsMargins(0, 0, 0, 0)
        gridLayout.setObjectName("gridLayout")
        self.web_view = QtWebKit.QWebView(tab)
        gridLayout.addWidget(self.web_view, 0, 0, 1, 1)
        self.main.tabWidget_files.addTab(tab, name)
        self.main.tabWidget_files.setCurrentWidget(tab)




#!/usr/bin/env python
#-*- coding: utf-8 -*-

import unittest
import os
import sys

from . import version
sys.path.append(os.path.join(os.getenv("PINGUINO_LIB"), "qtgui", "resources"))

from .qtgui.pinguino_core.pinguino import Pinguino, AllBoards
from .qtgui.pinguino_core.pinguino_config import PinguinoConfig
from .qtgui.pinguino_core.config import Config

Pinguino = Pinguino()
PinguinoConfig.set_environ_vars()
PinguinoConfig.check_user_files()
config = Config()
PinguinoConfig.update_pinguino_paths(config, Pinguino)
PinguinoConfig.update_pinguino_extra_options(config, Pinguino)
PinguinoConfig.update_user_libs(Pinguino)


########################################################################
class TestPreprocess(unittest.TestCase):

    #----------------------------------------------------------------------
    def test_delete_comments(self):

        cases = (

            ("//Pinguino Rules!", ""),
            ("/*Pinguino Rules!*/", ""),
            ("/*Pinguino //Rules!*/", ""),
            ("///*Pinguino Rules!*/", ""),

            ("\n".join([
            "#define LED1 0\n",
            "//#define LED2 1\n",
            "/*\n",
            "1\n",
            "2\n",
            "3\n",
            "*/\n",
            "#include <math.h>\n",
            ]),
            "\n".join([
            "#define LED1 0\n",
            "\n",
            "\n",
            "\n",
            "\n",
            "\n",
            "\n",
            "#include <math.h>\n",
            ]),
            )

        )

        for case in cases:
            got = Pinguino.remove_comments(case[0])
            expected = case[1]
            self.assertMultiLineEqual(got, expected,
                                      "Remove comments: Failure\ngot: '{}'\nexpected: '{}'".format(got, expected))


    #----------------------------------------------------------------------
    @classmethod
    def preprocess(cls, lib, libinstructions):

        def inter(self):
            got, d = Pinguino.replace_word(lib["pinguino"], libinstructions)
            self.assertEqual(got, lib["c"],
                             "Preprocess: Failure\ngot: '{}'\nexpected: '{}'".format(got, lib["c"]))
        return inter



########################################################################
class TestBareMinumumCompilation(unittest.TestCase):

    #----------------------------------------------------------------------
    @classmethod
    def compilation(cls, board):

        code = "void setup(){}; void loop(){}"

        def inter(self):

            try:
                Pinguino.set_board(board)
            except BaseException as msg:
                raise BaseException("Compilation: imposible set board {}\n{}".format(board.name, str(msg)))

            if board.arch == 8:
                for key in Pinguino.dict_boot.keys():
                    boot = Pinguino.dict_boot[key]
                    Pinguino.set_bootloader(*boot)
                    try:
                        Pinguino.compile_string(code)
                    except BaseException as msg:
                        self.fail("Compilation: impossible compile for {}, {}-bit, boot:{}\n{}".format(board.name, board.arch, key, str(msg)))

            if board.arch == 32:
                try:
                    Pinguino.compile_string(code)
                except BaseException as msg:
                    self.fail("Compilation: impossible compile for {}, {}-bit\n{}".format(board.name, board.arch, str(msg)))

        return inter


libs8 = Pinguino.get_regobject_libinstructions(8)
for lib in libs8:
    test_name = "test_preprocess_8_{}".format(lib["pinguino"].replace(".", "_"))
    setattr(TestPreprocess, test_name, TestPreprocess.preprocess(lib, libs8))


libs32 = Pinguino.get_regobject_libinstructions(8)
for lib in libs32:
    test_name = "test_preprocess_32_{}".format(lib["pinguino"].replace(".", "-"))
    setattr(TestPreprocess, test_name, TestPreprocess.preprocess(lib, libs32))


for board in AllBoards:
    test_name = "test_compile_{}".format(board.name.replace(" ", "_").replace(".", "_"))
    setattr(TestBareMinumumCompilation, test_name, TestBareMinumumCompilation.compilation(board))


if __name__ == "__main__":
    unittest.main()



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file '/home/yeison/Documentos/Desarrollo/Pinguino/GitHub/pinguino-ide/pinguino/qtgui/frames/paths.ui'
#
# Created: Tue Nov 17 17:51:44 2015
#      by: pyside-uic 0.2.15 running on PySide 1.2.4
#
# WARNING! All changes made in this file will be lost!

from PySide import QtCore, QtGui

class Ui_Paths(object):
    def setupUi(self, Paths):
        Paths.setObjectName("Paths")
        Paths.resize(737, 152)
        self.gridLayout_5 = QtGui.QGridLayout(Paths)
        self.gridLayout_5.setObjectName("gridLayout_5")
        self.horizontalLayout_2 = QtGui.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        spacerItem = QtGui.QSpacerItem(304, 23, QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Minimum)
        self.horizontalLayout_2.addItem(spacerItem)
        self.pushButton_close = QtGui.QPushButton(Paths)
        self.pushButton_close.setMinimumSize(QtCore.QSize(165, 0))
        self.pushButton_close.setDefault(True)
        self.pushButton_close.setObjectName("pushButton_close")
        self.horizontalLayout_2.addWidget(self.pushButton_close)
        self.gridLayout_5.addLayout(self.horizontalLayout_2, 3, 0, 1, 1)
        self.label = QtGui.QLabel(Paths)
        self.label.setObjectName("label")
        self.gridLayout_5.addWidget(self.label, 4, 0, 1, 1)
        self.horizontalLayout_3 = QtGui.QHBoxLayout()
        self.horizontalLayout_3.setSpacing(0)
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        self.label_6 = QtGui.QLabel(Paths)
        self.label_6.setObjectName("label_6")
        self.horizontalLayout_3.addWidget(self.label_6)
        self.lineEdit_pinguino_8_libs = QtGui.QLineEdit(Paths)
        self.lineEdit_pinguino_8_libs.setObjectName("lineEdit_pinguino_8_libs")
        self.horizontalLayout_3.addWidget(self.lineEdit_pinguino_8_libs)
        self.pushButton_clear_8_libs = QtGui.QPushButton(Paths)
        self.pushButton_clear_8_libs.setText("")
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/icons/icons/clear.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.pushButton_clear_8_libs.setIcon(icon)
        self.pushButton_clear_8_libs.setIconSize(QtCore.QSize(24, 24))
        self.pushButton_clear_8_libs.setFlat(True)
        self.pushButton_clear_8_libs.setObjectName("pushButton_clear_8_libs")
        self.horizontalLayout_3.addWidget(self.pushButton_clear_8_libs)
        self.pushButton_pinguino_8_libs = QtGui.QPushButton(Paths)
        self.pushButton_pinguino_8_libs.setObjectName("pushButton_pinguino_8_libs")
        self.horizontalLayout_3.addWidget(self.pushButton_pinguino_8_libs)
        self.gridLayout_5.addLayout(self.horizontalLayout_3, 0, 0, 1, 1)
        self.horizontalLayout_5 = QtGui.QHBoxLayout()
        self.horizontalLayout_5.setSpacing(0)
        self.horizontalLayout_5.setObjectName("horizontalLayout_5")
        self.label_8 = QtGui.QLabel(Paths)
        self.label_8.setObjectName("label_8")
        self.horizontalLayout_5.addWidget(self.label_8)
        self.lineEdit_pinguino_32_libs = QtGui.QLineEdit(Paths)
        self.lineEdit_pinguino_32_libs.setObjectName("lineEdit_pinguino_32_libs")
        self.horizontalLayout_5.addWidget(self.lineEdit_pinguino_32_libs)
        self.pushButton_clear_32_libs = QtGui.QPushButton(Paths)
        self.pushButton_clear_32_libs.setText("")
        self.pushButton_clear_32_libs.setIcon(icon)
        self.pushButton_clear_32_libs.setIconSize(QtCore.QSize(24, 24))
        self.pushButton_clear_32_libs.setFlat(True)
        self.pushButton_clear_32_libs.setObjectName("pushButton_clear_32_libs")
        self.horizontalLayout_5.addWidget(self.pushButton_clear_32_libs)
        self.pushButton_pinguino_32_libs = QtGui.QPushButton(Paths)
        self.pushButton_pinguino_32_libs.setObjectName("pushButton_pinguino_32_libs")
        self.horizontalLayout_5.addWidget(self.pushButton_pinguino_32_libs)
        self.gridLayout_5.addLayout(self.horizontalLayout_5, 1, 0, 1, 1)

        self.retranslateUi(Paths)
        QtCore.QMetaObject.connectSlotsByName(Paths)

    def retranslateUi(self, Paths):
        Paths.setWindowTitle(QtGui.QApplication.translate("Paths", "Paths", None, QtGui.QApplication.UnicodeUTF8))
        self.pushButton_close.setText(QtGui.QApplication.translate("Paths", "Close", None, QtGui.QApplication.UnicodeUTF8))
        self.label.setText(QtGui.QApplication.translate("Paths", "<html><head/><body><p>Pinguino IDE requires a restart to reinitialize its preferences.</p></body></html>", None, QtGui.QApplication.UnicodeUTF8))
        self.label_6.setText(QtGui.QApplication.translate("Paths", "8-bit libraries:", None, QtGui.QApplication.UnicodeUTF8))
        self.pushButton_pinguino_8_libs.setText(QtGui.QApplication.translate("Paths", "Change...", None, QtGui.QApplication.UnicodeUTF8))
        self.label_8.setText(QtGui.QApplication.translate("Paths", "32-bit libraries:", None, QtGui.QApplication.UnicodeUTF8))
        self.pushButton_pinguino_32_libs.setText(QtGui.QApplication.translate("Paths", "Change...", None, QtGui.QApplication.UnicodeUTF8))

import resources_rc



#!/usr/bin/env python
#-*- coding: utf-8 -*-

from PySide import QtGui

########################################################################
class CompleteIcons(object):


    #----------------------------------------------------------------------
    def __init__(self):

        self.iconFunction = QtGui.QIcon()
        self.iconFunction.addPixmap(QtGui.QPixmap(":/autocompleter/autocompleter/function.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)

        self.iconLibrary = QtGui.QIcon()
        self.iconLibrary.addPixmap(QtGui.QPixmap(":/autocompleter/autocompleter/library.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)

        self.iconBratches = QtGui.QIcon()
        self.iconBratches.addPixmap(QtGui.QPixmap(":/autocompleter/autocompleter/vectors.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)

        self.iconCuotation = QtGui.QIcon()
        self.iconCuotation.addPixmap(QtGui.QPixmap(":/autocompleter/autocompleter/cuotation.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)

        self.iconFloat = QtGui.QIcon()
        self.iconFloat.addPixmap(QtGui.QPixmap(":/autocompleter/autocompleter/float.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)

        self.iconInt = QtGui.QIcon()
        self.iconInt.addPixmap(QtGui.QPixmap(":/autocompleter/autocompleter/int.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)

        self.iconReserved = QtGui.QIcon()
        self.iconReserved.addPixmap(QtGui.QPixmap(":/autocompleter/autocompleter/reserved.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)

        self.iconBool = QtGui.QIcon()
        self.iconBool.addPixmap(QtGui.QPixmap(":/autocompleter/autocompleter/bool.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)

        self.iconByte = QtGui.QIcon()
        self.iconByte.addPixmap(QtGui.QPixmap(":/autocompleter/autocompleter/byte.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)

        self.iconDouble = QtGui.QIcon()
        self.iconDouble.addPixmap(QtGui.QPixmap(":/autocompleter/autocompleter/double.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)

        self.iconEnum = QtGui.QIcon()
        self.iconEnum.addPixmap(QtGui.QPixmap(":/autocompleter/autocompleter/enum.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)

        self.iconLong = QtGui.QIcon()
        self.iconLong.addPixmap(QtGui.QPixmap(":/autocompleter/autocompleter/long.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)

        self.iconShort = QtGui.QIcon()
        self.iconShort.addPixmap(QtGui.QPixmap(":/autocompleter/autocompleter/short.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)

        self.iconStruct = QtGui.QIcon()
        self.iconStruct.addPixmap(QtGui.QPixmap(":/autocompleter/autocompleter/struct.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)

        self.iconUnion = QtGui.QIcon()
        self.iconUnion.addPixmap(QtGui.QPixmap(":/autocompleter/autocompleter/union.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)

        self.iconWord = QtGui.QIcon()
        self.iconWord.addPixmap(QtGui.QPixmap(":/autocompleter/autocompleter/word.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)

        self.iconSnippet = QtGui.QIcon()
        self.iconSnippet.addPixmap(QtGui.QPixmap(":/autocompleter/autocompleter/snippet.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)

        self.iconDirectives = QtGui.QIcon()
        self.iconDirectives.addPixmap(QtGui.QPixmap(":/autocompleter/autocompleter/directivas.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)




#!/usr/bin/env python
#-*- coding: utf-8 -*-

import re
import os
from math import sqrt
from datetime import datetime

from PySide import QtGui, QtCore

from .blocks import Blocks, OPENHANDCURSOR, CLOSEHANDCURSOR
#from .constant import INTRO_CODE
from ..bloques import BlockLinear, BlockFunction, BlockNested, \
     BlockSpace, BlockNestedSecond, BlockSpaceBool, BlockFrameEdit
from ..py_bloques.get_blocks import all_sets
from ..py_bloques import constructor
from ...frames.select_area import Ui_Selection
from ...ide.methods.dialogs import Dialogs
from ...ide.methods.decorators import Decorator

#TEMPLATES = {"and": {"python":"and", "pinguino":"&&",},
             #"not": {"python":"not", "pinguino":"!",},
             #"or": {"python":"or", "pinguino":"||",},
             #}

exclude_auto_open = ["button_pressed", "button_released"]

########################################################################
class Metadata(object):
    pass

########################################################################
class WorkArea(QtGui.QWidget):

    #def __init__(self, parent, scroll, widget, frame, ide):
    def __init__(self, parent, scroll, frame, ide):

        super(WorkArea, self).__init__(parent)


        # self.PdmPhysicalDpiY.

        self.main = ide.main
        self.ide = ide
        self.frame = frame
        self.clearArea = frame.clear_area
        self.scroll = scroll
        self.add_group_blocks = frame.add_group_blocks

        self.SelectionAbs = []

        self.piecePixmaps = []
        self.pieceRects = []
        self.pieceLocations = []

        self.gridLayout = QtGui.QGridLayout(parent)
        self.gridLayout.addWidget(self, 0, 0, 1, 1)
        self.gridLayout.setContentsMargins(1, 0, 0, 0)
        self.gridLayout.setSpacing(0)

        self.isDragging = False
        self.isSelecting = False
        self.isOpening = False
        self.hasSelection = False
        self.isConnecting = False
        self.CHILD = None
        self.POS = QtCore.QPoint(0, 0)

        self.prepareAccept = None

        self.selFin = QtCore.QPoint(0, 0)

        self.setAcceptDrops(True)

        #self.constant_update()


        self.dicTypes = {str(type(0)): "int",
                         str(type(0.1)): "float",
                         str(type("")): "char",
                         }

        self.SelectArea = QtGui.QWidget(self)

        select = Ui_Selection()
        select.setupUi(self.SelectArea)

        self.SelectArea.hide()

        self.contextMenuEvent = self.build_menu


        self.update_user()


    #----------------------------------------------------------------------
    def build_menu(self, event):
        menu = QtGui.QMenu()

        if self.SelectArea.isVisible():
            #on selection
            menu.addAction(QtGui.QApplication.translate("Graphical", "Delete selected blocks"), self.dele_blocks)
            menu.addAction(QtGui.QApplication.translate("Graphical", "Take Screenshot from selected area"), self.screen_shot_area)

        else:
            #normal menu
            #menu.addAction(QtGui.QApplication.translate("Graphical", "Export code to pinguino editor"), self.export_code_to_pinguino_editor)
            menu.addAction(self.main.actionExport_code_to_editor)
            menu.addAction(self.main.actionInsert_Block)
            # menu.addAction(self.main.actionView_Pinguino_code)

        menu.setStyleSheet("""
        QMenu {
            font-family: inherit;
            font-weight: normal;
            }

        """)

        menu.exec_(event.globalPos())



    #-------------------------          ---------------------------------------------
    def export_code_to_pinguino_editor(self):

        code = self.update_code()

        # self.ide.switch_ide_mode(False)
        self.ide.ide_new_file()

        editor = self.ide.get_current_editor()
        editor.text_edit.setText(code)


    #----------------------------------------------------------------------
    def get_type_magnetic(self, child, parent=None):

        pos = []
        localType = child.metadata.type_
        if parent is None:

            for blk in self.get_project_blocks():
                blk.metadata.object_.updatePoints()


                if localType in blk.metadata.to_type.keys() and blk != child:
                    for i in blk.metadata.to_type[localType]:

                        if child.metadata.type_ in ["tipo5", "tipo2"]:
                            _pos_ = self.mapFromGlobal(i)


                        else: _pos_ = i+blk.pos()

                        pos.append([_pos_, blk])

        else:
            blk = parent
            blk.metadata.object_.updatePoints()

            if localType in blk.metadata.to_type.keys() and blk != child:
                for i in blk.metadata.to_type[localType]:

                    if child.metadata.type_ in ["tipo5", "tipo2"]:
                        _pos_ = self.mapFromGlobal(i)

                    else: _pos_ = i+blk.pos()

                    pos.append([_pos_, blk])


        return pos


    #----------------------------------------------------------------------
    def get_better_pos(self, posList, pos, child):

        distancia = lambda pa, pb:sqrt((pa.x()-pb.x())**2+(pa.y()-pb.y())**2)

        if not posList: return pos, False, "None"

        newPosList = []
        for point in posList:

            s = point[0]
            b = point[1]

            if b.metadata.type_ == "tipo1":
                if b.metadata.to:
                    if not child.metadata.type_ in "tipo2 tipo5".split(): continue

            elif b.metadata.type_ in "tipo4".split():
                if b.metadata.nested:
                    if not child.metadata.type_ in "tipo2 tipo5".split(): continue

            elif b.metadata.type_ in "tipo7 tipo9".split():
                if b.metadata.to and b.metadata.nested:
                    if not child.metadata.type_ in "tipo2 tipo5".split(): continue

            h = distancia(s, pos)
            if h < 20 and h != 0.0: newPosList.append((s, h, point[1]))

        if not newPosList: return pos, False, "None"

        h2 = 100
        for p in newPosList:
            if p[1] < h2:
                h2 = p[1]
                poin = p[0]
                #ID = p[2]

        return poin, True, p[2]


    #----------------------------------------------------------------------
    def new_bloq(self, name, args, pos, baseName, full=None):

        newIcon = QtGui.QWidget(self)

        # newIcon = QtGui.QPaintEngine(self)

        newIcon.NAME = name
        newIcon.ARGS = args
        newIcon.BASENAME = baseName

        nuevo = self.build_block(name, newIcon, args)
        nuevo.name = name
        nuevo.ContexMenu = True

        newIcon.metadata = Metadata()

        newIcon.metadata.to = []
        newIcon.metadata.nested = []
        newIcon.metadata.from_ = []
        newIcon.metadata.inside = []
        newIcon.metadata.pos_ = pos
        newIcon.metadata.parent = "None"

        newIcon.metadata.name = name
        newIcon.metadata.basename = baseName
        newIcon.metadata.widget = newIcon

        newIcon.metadata.type_ = nuevo.Type
        newIcon.metadata.object_ = nuevo
        newIcon.metadata.to_type = nuevo.toType

        newIcon.metadata.expandible = nuevo.expandible
        newIcon.metadata.expand = nuevo.f_expand

        newIcon.metadata.add_parent = nuevo.addParent
        newIcon.metadata.remove_parent = nuevo.removeParent

        newIcon.metadata.size = nuevo.size

        newIcon.metadata.line_code = nuevo.getLine
        newIcon.metadata.get_contructor = nuevo.getConstructor

        newIcon.metadata.code_start = nuevo.codeStart

        newIcon.metadata.destroy_this = nuevo.destroy_this

        pos2 = self.mapFromGlobal(pos)

        newIcon.move(pos2)
        newIcon.show()

        if full:
            if full == [False]: pass
            else: self.fill_widget_with(newIcon, full)
        else: self.fill_widget_with(newIcon, nuevo.fullWidgetWith)

        return newIcon, pos2


    #----------------------------------------------------------------------
    def fill_widget_with(self, parent, content):

        for block in content:
            if type(block) == tuple:
                block, value = block
            else:
                value = None

            name = all_sets.get(block.replace("-", "_"), None)
            if name is None: return
            else: name = name[:]
            NAME = name[0]
            ARGS = name
            BASENAME = block
            child = self.new_bloq(NAME, ARGS, QtCore.QPoint(), BASENAME)[0]

            if value:
                child.metadata.widget.set_value(value)

            child.metadata.add_parent([parent.metadata.widget, child.metadata.widget], force=True)
            child.metadata.from_.append(parent)



    #----------------------------------------------------------------------
    def mouseMoveEvent(self, event=None, eventPos=None, child=None, pos=QtCore.QPoint(0, 0)):

        self.isDragging = True
        if event != None:
            eventPos = event.pos()
            self.selFin = eventPos
            self.set_selection()

        self.repaint()

        if child == None:
            child, pos = self.get_child_drag()


        if child != None:

            while child.objectName() != "Bloq!": child = child.parent()

            if child.parent() != self:

                child.setParent(self)

                child.move(eventPos + QtCore.QPoint(-5, -20))
                child.metadata.pos_ = eventPos
                child.metadata.object_.removeParent()
                child.show()
                self.mousePressEvent(event)
                return

            else:
                point = eventPos - pos




            listPos = self.get_type_magnetic(child)

            point, accept, parent = self.get_better_pos(listPos, point, child)

            self.prepareAccept = [accept, child, parent]
            self.accept_move(False, child, parent)


            relative = point - child.pos()
            child.move(point)

            self.move_group(child, point - child.metadata.pos_)


            self.SelectArea.show()
            under = self.SelectionAbs[:]

            if self.SelectArea.isVisible() and (child in under):

                #self.SelectArea.hide()

                if len(under) > 1:
                    self.SelectArea.show()
                    selected = under
                    #if child in selected: selected.remove(child)
                    selected.remove(child)
                    for Id in selected:
                        if Id.metadata.from_ == []:
                            #print Id

                            npos = relative + Id.metadata.widget.pos()
                            Id.metadata.widget.move(npos)
                            Id.metadata.pos_ = npos
                            self.move_group(Id, relative)
                            #self.moveGroupInside(Id, relative)
                    self.SelectArea.move(self.SelectArea.pos()+relative)

            else:
                self.SelectArea.hide()


            child.metadata.pos_ = point

            globalMax = self.scroll.mapToGlobal(QtCore.QPoint(self.scroll.size().width(), self.scroll.size().height()-40))
            globalMin = self.scroll.mapToGlobal(self.scroll.pos())
            globalPoint = self.mapToGlobal(point)

            if event != None:

                if globalPoint.y() < globalMin.y():
                    value = self.scroll.verticalScrollBar().value() - 10
                    self.scroll.verticalScrollBar().setValue(value)
                elif globalPoint.y() > globalMax.y():
                    value = self.scroll.verticalScrollBar().value() + 10
                    self.scroll.verticalScrollBar().setValue(value)

                if globalPoint.x() < globalMin.x():
                    value = self.scroll.horizontalScrollBar().value() - 10
                    self.scroll.horizontalScrollBar().setValue(value)
                elif globalPoint.x() > globalMax.x():
                    value = self.scroll.horizontalScrollBar().value() + 10
                    self.scroll.horizontalScrollBar().setValue(value)


    #----------------------------------------------------------------------
    def set_selection(self):

        if self.isSelecting:
            s = self.selFin - self.selInicio
            x = s.x()
            y = s.y()
            if x > 0 and y > 0: self.SelectArea.move(self.selInicio)
            if x < 0 and y > 0: self.SelectArea.move(self.selFin.x(), self.selInicio.y())
            if x < 0 and y < 0: self.SelectArea.move(self.selFin)
            if x > 0 and y < 0: self.SelectArea.move(self.selInicio.x(), self.selFin.y())

            self.SelectArea.resize(QtCore.QSize(abs(x), abs(y)))

            #self.SelectArea.raise_()
            self.SelectArea.lower()
            self.SelectArea.show()
            #self.shadownBlock()
            self.hasSelection = True


    #----------------------------------------------------------------------
    def accept_move(self, accept, child, _ID=None, nested=None):

        if accept:
            #print point
            child.metadata.from_.append(_ID)
            child.metadata.add_parent([_ID.metadata.widget, child.metadata.widget])

            if nested is None:
                nested = False
                if _ID.metadata.type_ in ["tipo4", "tipo7", "tipo9"]:
                    nested = _ID.metadata.object_.addSubs([_ID.metadata.widget, child.metadata.widget])


            if nested:
                if not child in _ID.metadata.nested:
                    _ID.metadata.nested.append(child)
                    if child in _ID.metadata.to:
                        _ID.metadata.to.remove(child)

            else:
                if child.metadata.type_ in ["tipo5", "tipo2"]:
                    if not child in _ID.metadata.inside:
                        _ID.metadata.inside.append(child)
                else:
                    if not child in _ID.metadata.to:
                        _ID.metadata.to.append(child)
                        if child in _ID.metadata.nested:
                            _ID.metadata.nested.remove(child)

        else:

            child.metadata.from_ = []
            for blk in self.get_project_blocks():
                if child in blk.metadata.to:
                    blk.metadata.to.remove(child)

                if child in blk.metadata.inside:
                    blk.metadata.inside.remove(child)

                if child in blk.metadata.nested:
                    blk.metadata.nested.remove(child)

                    break




    #----------------------------------------------------------------------
    def expand_all(self):

        for block in self.get_project_blocks():
            if block.metadata.expandible:
                size1 = block.metadata.widget.size().height()
                try: size = self.get_height(block, self.get_metadata_nested)
                except RuntimeError: continue
                block.metadata.expand(size)
                size2 = block.metadata.widget.size().height()
                s = size2 - size1
                #if release:
                if not self.isOpening:
                    self.move_group(block, QtCore.QPoint(0, s), ignore_nested=True)

        if self.isOpening: self.isOpening = False


    #----------------------------------------------------------------------
    def get_height(self, block, func):

        size = 0
        #listIDs = self.getNested(ID)
        listIDs = func(block)
        #if func2 != None: listIDs.extend(func2(ID))

        for blk in listIDs:
            if blk.metadata.type_ == "tipo7":
                size += blk.metadata.widget.height() - 7 - 9

            elif blk.metadata.expandible:
                #size += self.getWidget(ID).height() - 7
                size += blk.metadata.widget.height() - 7

            else:
                size += blk.metadata.size
            size += self.get_height(blk, self.get_metadata_to)
        return size


    #----------------------------------------------------------------------
    def move_group(self, child, point, ignore_nested=False):

        if not ignore_nested:
            IDs = self.get_metadata_to(child) + self.get_metadata_nested(child)
        else:
            IDs = self.get_metadata_to(child)


        if not IDs: return

        toMove = []
        toMove.extend(IDs)
        while IDs:
            nIDs = []
            for ID in IDs:
                if ID: nIDs.extend(self.get_metadata_to(ID) + self.get_metadata_nested(ID))
            toMove.extend(nIDs)
            IDs = nIDs

        for ID in toMove:
            #ID.metadata.widget.move(ID.metadata.widget.pos() + point)
            ID.move(ID.pos() + point)
            #print self.getWidget(ID).pos() + point
            ID.metadata.pos_ = ID.pos()  # + point



    #----------------------------------------------------------------------
    def get_metadata_to(self, ID):
        return ID.metadata.to

    #----------------------------------------------------------------------
    def get_metadata_nested(self, ID):
        return ID.metadata.nested


    #----------------------------------------------------------------------
    def mousePressEvent(self, event):

        if event.button() == QtCore.Qt.RightButton: return
        self.hasSelection = False
        if len(self.getUnderSelection()) == 0:
            self.SelectArea.resize(QtCore.QSize(0, 0))

        self.SelectArea.hide()
        self.forExpand = True
        child = self.childAt(event.pos())

        if child != None:

            editor = self.ide.get_tab()
            if not editor.tabText(editor.currentIndex()).endswith("*"):
                editor.setTabText(editor.currentIndex(), editor.tabText(editor.currentIndex())+"*")
            self.isSelecting = False
            while child.objectName() != "Bloq!":
                child = child.parent()
                if child == None:
                    child.sendBack()
                    return

            if not child.childAt(child.mapFromGlobal(event.globalPos())):
                return


            #if not child.metadata.type_ in "tipo4 tipo9 tipo7".split():
                #child.raise_()


            pos = child.pos()
            self.global_close_hand = [child.childAt(child.mapFromGlobal(event.globalPos()))]

            if event.modifiers() == QtCore.Qt.ControlModifier:
                #Copy full block
                if child.metadata.parent != "None":
                    pos += child.metadata.parent.metadata.widget.pos()
                fullw = map(lambda wdg:wdg.metadata.basename, child.metadata.inside)
                child.childAt(child.mapFromGlobal(event.globalPos())).setCursor(CLOSEHANDCURSOR)
                child = self.new_bloq(child.NAME, child.ARGS, self.mapToGlobal(pos), child.BASENAME, full=fullw)[0]


            if event.modifiers() == QtCore.Qt.ShiftModifier:
                #Copy a basic block
                if child.metadata.parent != "None":
                    pos += child.metadata.parent.metadata.widget.pos()
                child.childAt(child.mapFromGlobal(event.globalPos())).setCursor(CLOSEHANDCURSOR)
                child = self.new_bloq(child.NAME, child.ARGS, self.mapToGlobal(pos), child.BASENAME, full=[False])[0]


            if child.childAt(child.mapFromGlobal(event.globalPos())):
                child.childAt(child.mapFromGlobal(event.globalPos())).setCursor(CLOSEHANDCURSOR)

            self.set_child_drag(child, event.pos()-pos)



        else:
            self.selInicio = event.pos()
            self.isSelecting = True



    #----------------------------------------------------------------------
    def mouseReleaseEvent(self, event):

        self.isDragging = False

        if self.CHILD != None:
            if self.CHILD.childAt(self.CHILD.mapFromGlobal(event.globalPos())):
                self.CHILD.childAt(self.CHILD.mapFromGlobal(event.globalPos())).setCursor(OPENHANDCURSOR)
                for ch in self.global_close_hand:
                    ch.setCursor(OPENHANDCURSOR)

            self.frame.main.actionSave_file.setEnabled(True)

        if self.getUnderSelection() == []:
            self.SelectArea.hide()
            self.SelectionAbs = []
        else: self.SelectionAbs = self.getUnderSelection()


        if self.prepareAccept != None:
            self.accept_move(*self.prepareAccept)


        self.CHILD = None
        self.prepareAccept = None
        self.expand_all()

        #self.paintEvent()


    #----------------------------------------------------------------------
    def set_child_drag(self, child, pos):

        self.CHILD = child
        self.POS = pos

    #----------------------------------------------------------------------
    def get_child_drag(self):

        return self.CHILD, self.POS

    #----------------------------------------------------------------------
    #@Decorator.requiere_no_dragging()
    def constant_update(self):

        for block in self.get_project_blocks():
            block.metadata.widget.adjustSize()
        self.expand_all()


    #----------------------------------------------------------------------
    #@Decorator.requiere_no_dragging()
    def auto_raise(self):

        on_sort = []
        for block in self.get_project_blocks():
            on_sort.append((block.metadata.object_.widget.size().height(), block))

        on_sort.sort(key=lambda li: li[0])
        for h, block in on_sort:
            block.lower()

        self.SelectArea.lower()


    #----------------------------------------------------------------------
    def paintEvent(self, event=None):
        self.painter = QtGui.QPainter()
        self.painter.begin(self)
        self.painter.fillRect(self.rect(), QtCore.Qt.white)

        self.draw_grid(QtGui.QColor("#e6eef4"), self.painter, 15)

        self.painter.end()


    #----------------------------------------------------------------------
    def draw_grid(self, color, painter, space):

        painter.setPen(QtGui.QPen(QtGui.QBrush(QtGui.QColor(color)), 1, QtCore.Qt.DashLine))

        #j = 0
        for i in range(0, self.size().height(), space):
            painter.drawLine(QtCore.QPoint(0, i), QtCore.QPoint(self.size().height(), i))
            #j += 1

        #j = 0
        for i in range(0, self.size().width(), space):
            painter.drawLine(QtCore.QPoint(i, 0), QtCore.QPoint(i, self.size().width()))
            #j += 1


    #----------------------------------------------------------------------
    def get_code_start(self):

        codepi = ""
        #codepy = ""
        for block in self.get_project_blocks():
            if block.metadata.code_start:

                codepi += block.metadata.code_start.get("pinguino", "")[0]
                #codepy += block.metadata.code_start.get("python", "")

                func, type_ = block.metadata.code_start.get("pinguino", ["", ["", ""]])[1]
                if func: self.dicTypes[func] = type_

        #return codepi, codepy
        return codepi

    #----------------------------------------------------------------------
    def update_code(self):

        self.allVars = []
        pinguino_code = ""

        pi = self.get_code_start()
        pinguino_code += pi

        for block in self.get_project_blocks():
            if block.metadata.basename == "global_vars":
                pinguino_code += "//  "
                pinguino_code += self.get_code_from(block).replace("{\n", "").replace("}\n", "").replace("    ", "")

        #gloval_vars = ""
        #if len(self.allVars) > 0: gloval_vars = "global " + ", ".join(self.allVars)

        funcs = []
        for section in ["user_function", "interfaz_function", "setup", "loop", "main_py"]:

            for key in self.get_project_blocks():
                if key.metadata.basename == section :

                    funcs.append(section)

                    self.listTypeVars = {}
                    code = self.get_code_from(key).replace("\n{\n", "{\n")

                    if code == section + "()\n": code = section + "(){}\n"

                    if self.function_has_return(key):

                        #If function has return, get it, else false
                        ret = self.function_has_return(key, ret=True)

                        typeVar = self.listTypeVars.get(ret, "")

                        if not typeVar:
                            ret = ret.replace("\"", "").replace("'", "").replace(" ", "")
                            if ret.isdigit(): typeVar = "int"
                            elif ret.count(".") == 1 and ret.replace(".", "").isdigit(): typeVar = "float"
                            elif ret.isalpha() and len(ret) == 1: typeVar = "char"
                            elif ret.isalpha(): typeVar = "char*"

                        pinguino_code += typeVar + " "
                        self.dicTypes[code[:code.find("(")]] = typeVar

                    pinguino_code += code.replace("(%s)", "()")

                    #pinguino_code += "\n"

        pinguino_code_extra = ""
        for f in ["loop", "setup"]:
            if not f in funcs:
                pinguino_code_extra += "\n" + f + "(){\n    }\n"

        #for key in TEMPLATES.keys():
            #pinguino_code = pinguino_code.replace("@"+key+"@", TEMPLATES[key]["pinguino"])

        pinguino_code += pinguino_code_extra

        global_ = ""
        if pinguino_code.find("__i__") > 0: global_ += "int __i__ = 0;\n"
        if pinguino_code.find("__j__") > 0: global_ += "int __j__ = 0;\n"

        if global_: global_ = "//  blocks variables, don't edit this\n" + global_ + "\n"

        header_code = """//------------------------------------------------------------------
// Pinguino source code generated automatically.
//
// Created: {}
// by: {}
//
//  WARNING! All changes made in this file will be lost!
//------------------------------------------------------------------\n
""".format(datetime.now().strftime("%Y-%m-%d %H:%M:%S"), os.getenv("PINGUINO_FULLNAME"))

        pinguino_code = header_code + global_ + pinguino_code

        return pinguino_code



    #----------------------------------------------------------------------
    def count_indent(self, linea):

        indent = 0
        for letra in linea:
            if letra != " ": return indent * " "
            indent += 1
        return indent * " "


    #----------------------------------------------------------------------
    def get_code_from(self, ID, auto_open=True):

        code = ""
        #incrementar = False
        if ID.metadata.basename == "asign":
            tempCode = ID.metadata.line_code().replace("\n", "").replace("%s", "1")
            nameVar = tempCode[:tempCode.find("=")]
            if not nameVar in self.allVars:
                self.allVars.append(nameVar)
                #try:
                if tempCode[tempCode.find("=")+1:-1].replace(" ", "") in ["True", "False"]:
                    code += "char "
                else:
                    try:
                        typeVar = self.dicTypes[tempCode[tempCode.find("=")+1:tempCode.find("(")]]
                        nameVar = tempCode[tempCode.find("=")+1:-1]

                    except KeyError:
                        try:
                            #typeVar = self.dicTypes[str(type(eval(tempCode[tempCode.find("=")+1:-1])))]
                            typeVar = self.dicTypes[str(type(eval(tempCode[tempCode.find("=")+1:tempCode.find("(")])))]
                            if typeVar == "char":
                                vvar = tempCode[tempCode.find("=")+1:tempCode.find("(")].replace("\"", "").replace("'", "")
                                if len(vvar) > 1: typeVar = "char*"
                            nameVar = tempCode[tempCode.find("=")+1:-1]

                        except NameError:
                            typeVar = ""

                    except:
                        typeVar = self.dicTypes["<type '" + tempCode[tempCode.find("=")+1:tempCode.find(" ")] + "'>"]
                        nameVar = tempCode[tempCode.find("=")+1:tempCode.find(" ")]

                    code += typeVar + " "
                    code += ID.metadata.line_code().replace("=", " = ", 1)

                try: self.listTypeVars[tempCode[:tempCode.find("=")]] = typeVar
                except: pass


            else:
                code += ID.metadata.line_code().replace("=", " = ", 1)



        else: code += self.fix_syntax_code(ID.metadata.line_code())

        if len(ID.metadata.nested) > 0:
            bloque = ID.metadata.nested[0]

            #if auto_open: code += "{"
            auto_open = not ID.metadata.basename in exclude_auto_open
            #print bloque.metadata.basename, exclude_auto_open
            if auto_open: code += "{"

            #if not incrementar:
            code += "\n" + "\n".join(map(lambda x:"    "+x, self.get_code_from(bloque, auto_open=auto_open).split("\n"))) + "}\n\n"
            #else:
                #code += "\n" + "\n".join(map(lambda x:"    "+x, self.get_code_from(bloque, auto_open=auto_open).split("\n"))) + "%s;}\n\n" % line[2]

        elif ID.metadata.type_ in ["tipo9"]:
            code = code.replace("\n", "{}\n")


        if len(ID.metadata.to) > 0:
            bloque = ID.metadata.to[0]
            code += self.get_code_from(bloque)

        return code

    #----------------------------------------------------------------------
    def fix_syntax_code(self, code):

        #fix_syntax = "> < >= <= =".split()
        #for char in fix_syntax:
            #code = code.replace(char, " "+char+" ")
        return code


    #----------------------------------------------------------------------
    def get_name_function(self, ID):

        code = ID.metadata.get_contructor()
        return code[0][1]

    #----------------------------------------------------------------------
    def get_name_var(self, ID):

        code = ID.metadata.line_code()
        return code[:code.find("=")]

    #----------------------------------------------------------------------
    def get_count_vars_function(self, ID):

        code = ID.metadata.get_contructor()
        return code.count(["space"])

    #----------------------------------------------------------------------
    def function_has_return(self, ID, ret=False):

        code = self.get_code_from(ID).split("\n")
        for line in code:
            if bool(re.match(" *return", line)):
                if ret: return re.match(" *return (.*);", line).group(1)
                else: return True
        return False


    #----------------------------------------------------------------------
    def add_function(self, name, countVars, _return, prename="", ID__=None):

        if _return: tipo = "output"
        else: tipo = "linear"

        #tmp = constructor.Output("User")
        tmp = getattr(constructor, tipo.capitalize())("User")

        tmp.addSyntax(prename)
        tmp.addLabel(name)
        tmp.addSyntax("(")

        if ID__: tmp.addSyntax(ID__+", ")

        for var in range(countVars):
            tmp.addSpace()
            tmp.addSyntax(", ")

        tmp.addSyntax(")")

        return ["User", "call-function-%s"%name, tmp.getBlock()]


    #----------------------------------------------------------------------
    def expand_size(self, child):

        if child.metadata.type_ == "tipo8":
            child.metadata.object_.setSmallSize()

    #----------------------------------------------------------------------
    def add_variable(self, name):

        tmp = constructor.Output("User")
        tmp.addEdit(name, None, (255, 255, 255))

        return ["User", "call-var-%s"%name, tmp.getBlock()]

    #----------------------------------------------------------------------
    @Decorator.timer(1000)
    @Decorator.requiere_open_files()
    @Decorator.requiere_graphical_mode()
    @Decorator.requiere_main_focus()
    @Decorator.requiere_tools_tab("Blocks")
    def update_user(self):

        new_blocks = []

        for block in self.get_project_blocks():

            if block.metadata.basename == "user_function":
                name = self.get_name_function(block)
                new_blocks.append(self.add_function(name, self.get_count_vars_function(block), self.function_has_return(block)))

            elif block.metadata.basename == "asign":
                name = self.get_name_var(block)
                new_blocks.append(self.add_variable(name))


        if os.getenv("PINGUINO_PROJECT"):
            functions = self.ide.get_functions()
            for function in functions:
                if function["name"] in ["setup", "loop"]: continue
                name = function["name"]
                if not function["args"]:
                    args = 0
                else:
                    args = len(function["args"].split(","))
                return_ = not (function["return"] == "void")
                new_blocks.append(self.add_function(name, args, return_))


            variables = self.ide.get_variables()
            for variable in variables:
                new_blocks.append(self.add_variable(variable["name"]))

            directives = self.ide.get_directives()
            for directive in directives:
                new_blocks.append(self.add_variable(directive["name"]))

        if hasattr(self, "cache_blocks"):
            if self.cache_blocks != new_blocks:
                self.clearArea("User")
                self.add_group_blocks("User", new_blocks)
                return
            else:
                return

        self.cache_blocks = new_blocks
        self.clearArea("User")
        self.add_group_blocks("User", new_blocks)

    #----------------------------------------------------------------------
    def dele_blocks(self):

        IDs = self.getUnderSelection()
        for Id in IDs:
            Id.metadata.destroy_this()

        self.SelectArea.hide()
        self.SelectionAbs = []


    #----------------------------------------------------------------------
    def getUnderSelection(self, inside=True, nested=False):

        IDs = []

        if self.SelectArea.isVisible():
            a = self.SelectArea.pos()
            x1 = a.x()
            y1 = a.y()
            b = self.SelectArea.size()
            x2 = x1 + b.width()
            y2 = y1 + b.height()

            for block in self.get_project_blocks():
            #for key in self.MetaData.keys():
                pos = block.metadata.pos_
                if block.metadata.parent == "None":
                    if x1 <= pos.x() <= x2 and y1 <= pos.y() <= y2: IDs.append(block)


        if inside:
            extra = []
            for Id in IDs: extra.extend(Id.metadata.inside)
            IDs = list(set(IDs + extra))

        if nested:
            extra = []
            for Id in IDs: extra.extend(Id.metadata.nested)
            IDs = list(set(IDs + extra))

        return IDs

    ##----------------------------------------------------------------------
    #def getAllRelatedFromID(self, IDs):
        #

        #all_ = []

        #def getNear(Id):
            #allo = []
            #for ID in self.MetaData.keys():
                #colection = self.MetaData[ID]["nested"] + self.MetaData[ID]["inside"] + self.MetaData[ID]["to"] + self.MetaData[ID]["from"]
                #if Id in colection:
                    #if ID in self.Searched:
                        #allo.extend([ID]+colection)
                    #else:
                        #self.Searched.append(ID)
                        #allo.extend(getNear(ID)+colection)

            #return allo


        #for IDD in IDs:
            #all_.extend(getNear(IDD))

        #all_ = list(set(all_))

        #return all_

    ##----------------------------------------------------------------------
    #def remove_block(self, key):
        #
        #self.MetaData[key]["object"].destroy_this()

        #try:
            #for ins in self.MetaData[key]["inside"]:
                #self.MetaData[ins]["object"].DeletLater = True
        #except KeyError:
            #pass

        #for key_ in self.MetaData.keys():
            #if key in self.MetaData[key_]["to"]: self.MetaData[key_]["to"].remove(key)
            #if key in self.MetaData[key_]["inside"]: self.MetaData[key_]["inside"].remove(key)
            #if key == self.MetaData[key_]["from"]: self.MetaData[key_]["from"] = []
            #if key in self.MetaData[key_]["nested"]: self.MetaData[key_]["nested"].remove(key)

        #b = self.MetaData.pop(key)
        #del b
        ##self.saveHistorial()



    #----------------------------------------------------------------------
    def screen_shot_area(self):
        self.SelectArea.hide()
        image = QtGui.QPixmap.grabWidget(self,
                                       QtCore.QRect(self.SelectArea.pos().x(),
                                                    self.SelectArea.pos().y(),
                                                    self.SelectArea.width(),
                                                    self.SelectArea.height()))

        filename = self.ide.get_tab().tabText(self.ide.get_tab().currentIndex())
        filename = os.path.splitext(filename)[0] + ".png"
        filename = Dialogs.set_save_image(self.ide, filename)
        if filename: image.save(filename, "png")


    #----------------------------------------------------------------------
    def build_block(self, name, widget, instructions):

        return eval(Blocks[name])(widget, instructions[2:])


    #----------------------------------------------------------------------
    def get_project_blocks(self):

        all_ = self.get_recursive_project_blocks(self.children())
        #print "%$%", len(all_)
        return all_

    #----------------------------------------------------------------------
    def get_recursive_project_blocks(self, children):

        childs = []
        for child in filter(lambda obj:getattr(obj, "metadata", False), children):
            childs.append(child)
            childs.extend(self.get_recursive_project_blocks(self.get_widgets_from_layout(child.metadata.object_.layout)))
        return childs

    #----------------------------------------------------------------------
    def get_widgets_from_layout(self, layout):

        if layout is None: return []
        widgets = []
        for index in range(layout.count()):
            widgets.append(layout.itemAt(index).widget())
        return filter(lambda obj:getattr(obj, "metadata", False), widgets)


    #----------------------------------------------------------------------
    def is_graphical(self):
        """"""
        return self.ide.is_graphical()

    #----------------------------------------------------------------------
    def get_tab(self):
        """"""
        return self.ide.get_tab()



#!/usr/bin/env python

import os
import sys
import shutil

from . import version
from .qtgui.pinguino_core.pinguino_config import PinguinoConfig

os.environ["PINGUINO_LIB"] = os.path.abspath(os.path.dirname(__file__))

PinguinoConfig.set_environ_vars()

#Check files and directories
PinguinoConfig.check_user_files()

#Remove files and directories
if os.path.isdir(os.path.join(os.getenv("PINGUINO_USER_PATH"), "source")):
    shutil.rmtree(os.path.join(os.getenv("PINGUINO_USER_PATH"), "source"))

if os.path.isdir(os.path.join(os.getenv("PINGUINO_USER_PATH"), "patches")):
    shutil.rmtree(os.path.join(os.getenv("PINGUINO_USER_PATH"), "patches"))

#Remove old files
#RB20150202 : each file must be checked before being deleted
if os.path.exists(os.path.join(os.getenv("PINGUINO_USER_PATH"), "reserved.pickle")):
    os.remove(os.path.join(os.getenv("PINGUINO_USER_PATH"), "reserved.pickle"))
if os.path.exists(os.path.join(os.getenv("PINGUINO_USER_PATH"), "pinguino.conf")):
    os.remove(os.path.join(os.getenv("PINGUINO_USER_PATH"), "pinguino.conf"))




#!/usr/bin/python

import sys
import os
import shutil

root = os.path.dirname(__file__)
files = os.listdir(".")
files = filter(lambda f:f.endswith(".svg"), files)

for size in ["16", "24", "32", "48"]:
    path = "../{}".format(size)

    if not os.path.exists(path):
        os.mkdir(path)
    else:
        shutil.rmtree(path)
        os.mkdir(path)


    for file in files:

        filename_svg = file
        filename_png = os.path.join(path, file.replace(".svg", ".png"))

        command = "inkscape {filename_svg} -e={filename_png} -C -w={size} -h={size}".format(**locals())

        print(command)
        os.system(command)


#!/usr/bin/env python
#-*- coding: utf-8 -*-

from .constructor import Kit

Kit = Kit()

########################################################################
class Communication(object):

    CDC_read = lambda n:Kit.addFunct("CDC", "CDC.read", 1, t="output", fill=[])
    CDC_write = lambda n:Kit.addFunct("CDC", "CDC.write", 1, t="linear", fill=["edit-c"])
    CDC_getstring = lambda n:Kit.addFunct("CDC", "CDC.getString", 0, t="output")
    CDC_getkey = lambda n:Kit.addFunct("CDC", "CDC.getKey", 0, t="output")
    CDC_print = lambda n:Kit.addFunct("CDC", "CDC.print", 1, t="linear", fill=["edit"])
    CDC_println = lambda n:Kit.addFunct("CDC", "CDC.println", 1, t="linear", fill=["edit"])
    CDC_printf = lambda n:Kit.addFunct("CDC", "CDC.printf", 2, t="linear", fill=["edit", "more-args"])

    Serial_available = lambda n:Kit.addFunct("Serial", "Serial.available", 0, t="output-bool", fill=[])
    Serial_init = lambda n:Kit.addFunct("Serial", "Serial.init", 1, t="linear", fill=["spin-int"])
    Serial_flush = lambda n:Kit.addFunct("Serial", "Serial.flush", 0, t="linear", fill=[])
    Serial_getKey = lambda n:Kit.addFunct("Serial", "Serial.getKey", 0, t="output", fill=[])
    Serial_getString = lambda n:Kit.addFunct("Serial", "Serial.getString", 0, t="output", fill=[])
    Serial_print = lambda n:Kit.addFunct("Serial", "Serial.print", 1, t="linear", fill=["edit"])
    Serial_println = lambda n:Kit.addFunct("Serial", "Serial.println", 1, t="linear", fill=["edit"])
    Serial_read = lambda n:Kit.addFunct("Serial", "Serial.read", 0, t="output", fill=[])
    Serial_write = lambda n:Kit.addFunct("Serial", "Serial.write", 1, t="linear", fill=["edit-c"])
    Serial_printf = lambda n:Kit.addFunct("Serial", "Serial.printf", 2, t="linear", fill=["edit", "more-args"])

    USB_read = lambda n:Kit.addFunct("USB", "USB.read", 0, t="output", fill=[])
    USB_write = lambda n:Kit.addFunct("USB", "USB.write", 1, t="linear", fill=["edit"])
    USB_printf = lambda n:Kit.addFunct("USB", "USB.printf", 2, t="linear", fill=["edit"])
    USB_available = lambda n:Kit.addFunct("USB", "USB.available", 0, t="output-bool", fill=[])
    USB_gets = lambda n:Kit.addFunct("USB", "USB.gets", 0, t="output", fill=[])
    USB_sets = lambda n:Kit.addFunct("USB", "USB.sets", 1, t="linear", fill=["edit-c"])




#!/usr/bin/env python
#-*- coding: utf-8 -*-

from . import constructor as Block
from .constructor import Kit

Kit = Kit()

########################################################################
class Servo(object):

    Servo_attach = lambda n:Kit.addFunct("Servo", "servo.attach", 1, fill=["spin-int"])
    Servo_detach = lambda n:Kit.addFunct("Servo", "servo.detach", 1, fill=["spin-int"])

    #----------------------------------------------------------------------
    def Servo_write(self):

        pm = Block.Linear("Servo")
        pm.addLabel("servo.write")
        pm.addSyntax("(")
        pm.addArg("SERVO")
        pm.addSyntax(",")
        pm.addDecorator(" ")
        pm.addArg("DEGREES")
        pm.addSyntax(")")
        pm.fillWith(["spin-int", "spin-int"])
        return pm.getBlock()


    #----------------------------------------------------------------------
    def Servo_read(self):

        pm = Block.Output("Servo")
        pm.addLabel("servo.read")
        pm.addSyntax("(")
        pm.addSpace()
        pm.addSyntax(")")
        pm.fillWith(["spin-int"])
        return pm.getBlock()


    #----------------------------------------------------------------------
    def Servo_setMaximumPulse(self):

        pm = Block.Linear("Servo")
        pm.addLabel("servo.setMaximumPulse")
        pm.addSyntax("(")
        pm.addArg("SERVO")
        pm.addSyntax(",")
        pm.addDecorator(" ")
        pm.addArg("MICROSEC")
        pm.addSyntax(")")
        pm.fillWith(["spin-int", "spin-int"])
        return pm.getBlock()


    #----------------------------------------------------------------------
    def Servo_setMinimumPulse(self):

        pm = Block.Linear("Servo")
        pm.addLabel("servo.setMinimumPulse")
        pm.addSyntax("(")
        pm.addArg("SERVO")
        pm.addSyntax(",")
        pm.addDecorator(" ")
        pm.addArg("MICROSEC")
        pm.addSyntax(")")
        pm.fillWith(["spin-int", "spin-int"])
        return pm.getBlock()



#!/usr/bin/env python
#-*- coding: utf-8 -*-

import os

from PySide import QtGui, QtCore


########################################################################
class Boards(object):

    def __init__(self):

        self.build_devices_arch()

        all_groups = [self.main.frame_arch,
                      self.main.frame_prgmode,
                      # self.main.frame_bootloader,
                      self.main.frame_devices_8,
                      self.main.frame_devices_32]

        #FIXME: add correct options
        self.HEAPSIZE = {"512 byte": 512,
                         "1024 byte": 1024,}

        self.main.comboBox_heapsize.clear()
        self.main.comboBox_heapsize.addItems(list(self.HEAPSIZE.keys()))

        self.OPTIMIZATION = "-O2 -O3 -Os".split()
        self.main.comboBox_optimization.clear()
        self.main.comboBox_optimization.addItems(self.OPTIMIZATION)
        self.init_groups()


        self.connect(self.main.radioButton_mode_bootloader, QtCore.SIGNAL("clicked()"), self.update_config)
        self.connect(self.main.radioButton_mode_icsp, QtCore.SIGNAL("clicked()"), self.update_config)

        self.connect(self.main.radioButton_compiler_sdcc, QtCore.SIGNAL("clicked()"), self.update_config)
        self.connect(self.main.radioButton_compiler_xc8, QtCore.SIGNAL("clicked()"), self.update_config)

        self.connect(self.main.radioButton_arch_8, QtCore.SIGNAL("clicked()"), self.update_config)
        self.connect(self.main.radioButton_arch_32, QtCore.SIGNAL("clicked()"), self.update_config)

        # self.connect(self.main.radioButton_bootloader_v1_v2, QtCore.SIGNAL("clicked()"), self.update_config)
        # self.connect(self.main.radioButton_bootloader_v4, QtCore.SIGNAL("clicked()"), self.update_config)

        self.load_config()

    #----------------------------------------------------------------------
    def update_mips16(self):
        self.configIDE.set("Board", "mips16", self.main.checkBox_mips16.isChecked())


    #----------------------------------------------------------------------
    def update_heapsize(self):
        self.configIDE.set("Board", "heapsize", self.main.checkBox_heapsize.isChecked())


    #----------------------------------------------------------------------
    def load_config(self):

        self.configIDE.load_config()

        arch = self.configIDE.config("Board", "arch", 8)
        self.main.radioButton_arch_8.setChecked(arch == 8)
        self.main.radioButton_arch_32.setChecked(arch == 32)
        self.main.frame_advance.setVisible(arch == 32)

        compiler = self.configIDE.config("Board", "compiler", "SDCC")  #SDCC default compiler
        self.main.radioButton_compiler_sdcc.setChecked(compiler=="SDCC")
        self.main.radioButton_compiler_xc8.setChecked(compiler=="XC8")

        mode = self.configIDE.config("Board", "mode", "bootloader")
        self.main.radioButton_mode_bootloader.setChecked(mode == "bootloader")
        self.main.radioButton_mode_icsp.setChecked(mode == "icsp")

        # bootloader = self.configIDE.config("Board", "bootloader", "v1_v2")
        # self.main.radioButton_bootloader_v1_v2.setChecked(bootloader == "v1_v2")
        # self.main.radioButton_bootloader_v4.setChecked(bootloader == "v4")

        mips16 = self.configIDE.config("Board", "mips16", True)
        self.main.checkBox_mips16.setChecked(mips16)

        heapsize = self.configIDE.config("Board", "heapsize", 512)
        heapsize_values = list(self.HEAPSIZE.values())
        index = heapsize_values.index(heapsize)
        self.main.comboBox_heapsize.setCurrentIndex(index)

        optimization = self.configIDE.config("Board", "optimization", "-O3")
        index = self.OPTIMIZATION.index(optimization)
        self.main.comboBox_optimization.setCurrentIndex(index)

        self.update_config()


    #----------------------------------------------------------------------
    def save_config(self):

        if self.main.radioButton_arch_8.isChecked(): arch = 8
        else: arch = 32
        self.configIDE.set("Board", "arch", arch)

        if self.main.radioButton_compiler_sdcc.isChecked(): compiler = "SDCC"
        else: compiler = "XC8"
        self.configIDE.set("Board", "compiler", compiler)

        if self.main.radioButton_mode_bootloader.isChecked(): mode = "bootloader"
        else: mode = "icsp"
        self.configIDE.set("Board", "mode", mode)

        # if self.main.radioButton_bootloader_v1_v2.isChecked(): bootloader = "v1_v2"
        # else: bootloader = "v4"
        # self.configIDE.set("Board", "bootloader", bootloader)

        name = self.configIDE.config("Board", "board_"+str(arch), None)
        self.configIDE.set("Board", "board", name)

        self.configIDE.set("Board", "mips16", self.main.checkBox_mips16.isChecked())
        heapsize = self.HEAPSIZE[self.main.comboBox_heapsize.currentText()]
        self.configIDE.set("Board", "heapsize", heapsize)
        self.configIDE.set("Board", "optimization", self.main.comboBox_optimization.currentText())

        self.configIDE.save_config()


    #----------------------------------------------------------------------
    def update_config(self):

        mode_boot = self.main.radioButton_mode_bootloader.isChecked()
        arch_8 = self.main.radioButton_arch_8.isChecked()
        # self.main.frame_bootloader.setVisible(mode_boot and arch_8)

        self.main.frame_devices_32.setVisible(not arch_8)
        self.main.frame_devices_8.setVisible(arch_8)
        self.main.frame_compiler.setVisible(arch_8)

        self.main.frame_advance.setVisible(not arch_8)
        self.save_config()
        self.status_info.setText(self.get_status_board())


    #----------------------------------------------------------------------
    def init_groups(self):

        arch_8 = self.main.radioButton_arch_8.isChecked()
        self.main.frame_devices_32.setVisible(not arch_8)
        self.main.frame_devices_8.setVisible(arch_8)


    #----------------------------------------------------------------------
    def set_board_name(self, name, arch):

        def dummy():
            self.configIDE.set("Board", "board_"+arch, name)
            self.update_config()
        return dummy


    #----------------------------------------------------------------------
    def build_devices_arch(self):

        #8bits
        name_checked = self.configIDE.config("Board", "board_8", "Pinguino 45k50")  #Pinguino 45k50 default board
        arch_8 = list(filter(lambda board:board.arch==8, self.pinguinoAPI._boards_))
        # arch_8.sort()

        count = 0
        side = 0  #left
        for board in arch_8:
            if arch_8.index(board) == (len(arch_8) // 2) + 1:
                count = 0
                side = 1  #rigth

            radio = QtGui.QRadioButton(self.main.frame_devices_8)
            self.main.gridLayout_device_8.addWidget(radio, count, side, 1, 1)
            radio.setText(board.name)
            radio.setToolTip(board.proc)
            if name_checked == board.name: radio.setChecked(True)
            self.connect(radio, QtCore.SIGNAL("clicked()"), self.set_board_name(board.name, "8"))
            count += 1


        #32bits
        name_checked = self.configIDE.config("Board", "board_32", "PIC32 Pinguino OTG")
        arch_32 = list(filter(lambda board:board.arch==32, self.pinguinoAPI._boards_))
        # arch_32.sort()

        count = 0
        side = 0  #left
        for board in arch_32:
            if arch_32.index(board) == (len(arch_32) // 2) + 1:
                count = 0
                side = 1  #rigth

            radio = QtGui.QRadioButton(self.main.frame_devices_32)
            self.main.gridLayout_device_32.addWidget(radio, count, side, 1, 1)
            radio.setText(board.name)
            radio.setToolTip(board.proc)
            if name_checked == board.name: radio.setChecked(True)
            self.connect(radio, QtCore.SIGNAL("clicked()"), self.set_board_name(board.name, "32"))
            count += 1




#!/usr/bin/env python
#-*- coding: utf-8 -*-

from . import constructor as Block

########################################################################
class Functions(object):

    #----------------------------------------------------------------------
    def repeat(self):

        pm = Block.Nested("Functions")
        pm.addSyntax("//")
        pm.addLabel("Repeat")
        pm.addSyntax("\n")
        pm.addSyntax("for(__i__ = 0; __i__ < ")
        pm.addSpace()
        pm.addSyntax("; __i__++)")
        pm.fillWith(["spin-int"])
        return pm.getBlock()


    #----------------------------------------------------------------------
    def button_pressed(self):

        pm = Block.Nested("Functions")
        pm.addSyntax("//")
        pm.addLabel("Button Pressed")
        pm.addSyntax("\n")
        pm.addSyntax("__j__ = ")
        pm.addSpace()
        pm.addSyntax(";\n")
        pm.addSyntax("if (digitalRead(__j__))\n")
        pm.addSyntax("{\n")
        pm.addSyntax("    while (digitalRead(__j__)){}")
        pm.fillWith(["spin-int"])
        pm.addContextHelp("Normally open")
        return pm.getBlock()


    #----------------------------------------------------------------------
    def button_released(self):

        pm = Block.Nested("Functions")
        pm.addSyntax("//")
        pm.addLabel("Button Released")
        pm.addSyntax("\n")
        pm.addSyntax("__j__ = ")
        pm.addSpace()
        pm.addSyntax(";\n")
        pm.addSyntax("if (!digitalRead(__j__))\n")
        pm.addSyntax("{\n")
        pm.addSyntax("    while (!digitalRead(__j__)){}")
        pm.fillWith(["spin-int"])
        pm.addContextHelp("Normally closed")
        return pm.getBlock()



#! /usr/bin/python2
#-*- coding: utf-8 -*-

import os
import logging

INFO = "{} is an optional dependence for Pinguino's Library Manager."

ErrorModules = {"gitpython": True,
                # "pysvn": True,
                "hgapi": True,
                }

try: import git
except ImportError:
    ErrorModules["gitpython"] = False
    logging.warning(INFO.format("gitpython"))

# try: import pysvn
# except ImportError:
    # ErrorModules["pysvn"] = False
    # logging.warning(INFO%"pysvn")

try: import hgapi
except ImportError:
    ErrorModules["hgapi"] = False
    logging.warning(INFO.format("hgapi"))


########################################################################
class Repository(object):

    #----------------------------------------------------------------------
    def __init__(self, repo_path, dir_path, repo_type):
        """"""
        self.repo_path = repo_path
        self.dir_path = dir_path

        self.REPOSITORIES = [
                             {"name": "git",
                              "file": ".git",
                              "cv": GitRepo},

                             {"name": "hg",
                              "file": ".hg",
                              "cv": HgRepo},

                             # {"name": "svn",
                              # "file": ".svn",
                              # "cv": SvnRepo},
                            ]

        self.set_repo(repo_type)


    #----------------------------------------------------------------------
    def search_repo(self):
        listdir = os.listdir(self.dir_path)

        for repo in self.REPOSITORIES:
            if repo["file"] in listdir:
                Repository = repo["cv"]()
        self.clone = Repository.clone
        self.update = Repository.update

    #----------------------------------------------------------------------
    def set_repo(self, repo_type):
        for repo in self.REPOSITORIES:
            if repo_type == repo["name"]:
                Repository = repo["cv"]()

        self.clone = Repository.clone
        self.update = Repository.update

    #----------------------------------------------------------------------
    def install_library(self):
        try:
            self.clone(self.repo_path, self.dir_path)
            return True
        except: return False

    #----------------------------------------------------------------------
    def update_library(self):
        try:
            self.search_repo()
            self.update(self.dir_path)
            return True
        except:
            return False



########################################################################
class GitRepo(object):

    #----------------------------------------------------------------------
    def clone(self, url, path):
        git.Repo.clone_from(url, path)

    #----------------------------------------------------------------------
    def update(self, path):
        if os.path.exists(os.path.join(path, ".git", "config.lock")): os.remove(os.path.join(path, ".git", "config.lock"))

        repo = git.Repo(path)
        repo.config_writer()
        head = repo.heads[0]
        head.checkout(force=True)
        remo = repo.remote()
        remo.pull()


########################################################################
class HgRepo(object):

    #----------------------------------------------------------------------
    def clone(self, url, path):
        hgapi.hg_clone(url, path)

    #----------------------------------------------------------------------
    def update(self, path):
        repo = hgapi.Repo(path)
        repo.hg_pull()
        repo.hg_update("default", clean=True)


# Since Google code down, this feature is not usefull
# ########################################################################
# class SvnRepo(object):

    # #----------------------------------------------------------------------
    # def clone(self, url, path):
        # client = pysvn.Client()
        # client.checkout(url, path)

    # #----------------------------------------------------------------------
    # def update(self, path):
        # client = pysvn.Client()
        # client.update(path)



#!/usr/bin/env python
#-*- coding: utf-8 -*-

import os
import re
import pickle

from PySide import QtGui, QtCore
from PySide.QtCore import QPoint

from .autocompleter import PinguinoAutoCompleter
from .autocomplete_icons import CompleteIcons
from .pinguino_highlighter import Highlighter
from .syntax import Autocompleter, Snippet, Helpers
#from ..methods import constants as Constants
#from ..methods.decorators import Decorator


########################################################################
class CustomTextEdit(QtGui.QTextEdit):

    #----------------------------------------------------------------------
    def __init__(self, parent, linenumber, highlighter, autocompleter):

        super(CustomTextEdit, self).__init__(parent)

        self.setLineWrapMode(self.NoWrap)

        with open(os.path.join(os.getenv("PINGUINO_USER_PATH"), "reserved.pickle"), "rb") as file_reserved:
            self.helpers = pickle.load(file_reserved).get("helpers", {})

        self.cursorC = QtGui.QCursor()
        self.linenumber = linenumber

        if autocompleter:
            self.completer = PinguinoAutoCompleter()
            self.completer.text_edit = self
            self.mousePressEvent = self.mouseAction
            self.completer.setFont(self.font())
            self.connect(self.completer, QtCore.SIGNAL("itemDoubleClicked(QListWidgetItem*)"), self.insertItem)
            self.completer.keyPressEvent = self.keyPressEvent_autocompleter
            self.completer.setFont(self.font())

            icons = CompleteIcons()
            self.completer.addItemsCompleter(Autocompleter["directive"], icons.iconDirectives)
            self.completer.addItemsCompleter(Autocompleter["reserved"], icons.iconReserved)
            self.completer.addItemsCompleter(Snippet.keys(), icons.iconSnippet)
            self.last_w = ""
            self.next_ignore = None



        if highlighter:
            Highlighter(self.document())

        self.setStyleSheet("""
        QTextEdit {
            background-color: #FFF;
            font-family: mono;
            font-weight: normal;
            font-size: 10pt;
            }
        """)



    #----------------------------------------------------------------------
    def mouseAction(self, event):

        if self.underMouse(): self.completer.hide()
        QtGui.QTextEdit.mousePressEvent(self, event)


    #----------------------------------------------------------------------
    def wheelEvent(self, event):

        if event.modifiers() == QtCore.Qt.ControlModifier:
            self.step_font_size(event.delta())
        else:
            super(CustomTextEdit, self).wheelEvent(event)


    #----------------------------------------------------------------------
    def step_font_size(self, delta):

        font = self.font()
        size = font.pointSize()
        if delta > 0: size = size + 1
        else: size = size - 1

        self.setStyleSheet("""
        QTextEdit {
            background-color: #FFF;
            font-family: mono;
            font-weight: normal;
            font-size: {}dpt;
            }

        """.format(size))


        self.linenumber.setStyleSheet("""
        font-family: mono;
        font-weight: normal;
        font-size: {}dpt;

        """.format(size))



    #----------------------------------------------------------------------
    def insertItem(self, completion):

        self.insert(completion.text())


    #----------------------------------------------------------------------

    def insert(self, completion):
        if not completion: return
        #selected = completion
        tc = self.textCursor()

        self.smart_under_selection(tc)

        tc.removeSelectedText()

        # if completion in Helpers.keys():

        self.temp_helpers = self.helpers.copy()
        self.temp_helpers.update(Helpers)
        self.temp_helpers.update(self.completer.local_functions)

        pos = tc.position()
        tc.movePosition(tc.EndOfLine, tc.KeepAnchor)
        at_right = tc.selectedText()
        tc.setPosition(pos, tc.MoveAnchor)

        if completion in self.temp_helpers.keys() and not at_right:
            pos = tc.position()

            # text_position = Snippet[completion].find("[!]")
            text_insert = self.temp_helpers[completion].replace("{{", "").replace("}}", "")
            position_in_line = tc.positionInBlock()

            start_position = self.temp_helpers[completion].find("{{")
            end_position = self.temp_helpers[completion].find("}}")

            tc.insertText(text_insert.replace("\n", "\n"+" "*position_in_line))
            tc.setPosition(pos + start_position)

            select = self.temp_helpers[completion][(start_position + 2):end_position]
            tc.beginEditBlock()
            self.moveCursor(tc.StartOfLine)
            self.find(select)
            tc.endEditBlock()


        elif completion in Snippet.keys():
            pos = tc.position()

            text_position = Snippet[completion].find("[!]")
            text_insert = Snippet[completion].replace("[!]", "")
            position_in_line = tc.positionInBlock()

            tc.insertText(text_insert.replace("\n", "\n"+" "*position_in_line))
            tc.setPosition(pos + text_position)

            self.setTextCursor(tc)

        elif re.match("(.+) +\[.+\]", str(completion)) != None:
            ins = re.match("(.+) +\[.+\]", str(completion)).group(1)
            tc.insertText(ins)

        else:
            tc.insertText(completion)

        self.completer.hide()
        self.setFocus()


    #----------------------------------------------------------------------

    def getPosPopup(self):
        pos = self.pos()
        pos1 = self.mapToGlobal(pos)
        pos2 = QPoint()
        cRect = self.cursorRect()
        pos2.setX(cRect.x())
        pos2.setY(cRect.y())
        return pos1 + pos2


    #----------------------------------------------------------------------

    def autoInsert(self, event):
        key = event.text()
        tc = self.textCursor()

        def accept(insert):
            selected = tc.selectedText()
            tc.insertText(key + selected + insert)
            tc.movePosition(tc.Left, tc.MoveAnchor)
            tc.setPosition(tc.position()-len(selected), tc.MoveAnchor)
            tc.setPosition(tc.position()+len(selected), tc.KeepAnchor)
            return tc

        if key == "[":
            self.next_ignore = "]"
            return accept("]")

        elif key == "{":
            self.next_ignore = "}"
            return accept("}")

        elif key == '"':
            self.next_ignore = '"'
            return accept('"')

        elif key == "'":
            self.next_ignore = "'"
            return accept("'")

        elif key == "(":
            self.next_ignore = ")"
            return accept(")")
        else: return False


    #----------------------------------------------------------------------
    def keyPressEvent_autocompleter(self, event):

        #desplazar
        if event.key() in (QtCore.Qt.Key_Up, QtCore.Qt.Key_Down) and self.completer.isVisible():
            self.completer.setFocus()
            if event.key() == QtCore.Qt.Key_Up: self.completer.up()
            elif event.key() == QtCore.Qt.Key_Down: self.completer.down()
            return

        #insertar
        elif event.key() in (QtCore.Qt.Key_Enter, QtCore.Qt.Key_Enter-1, QtCore.Qt.Key_Tab) and self.completer.isVisible():
            self.insert(self.completer.currentItem().text())
            self.completer.hide()
            return

        self.keyPressEvent(event)


    #----------------------------------------------------------------------
    def force_keyPressEvent(self, event):

        super(CustomTextEdit, self).keyPressEvent(event)


    #----------------------------------------------------------------------

    def __keyPressEvent__(self, event):
        self.setFocus()

        key_text = event.text()
        if self.next_ignore == key_text:
            self.next_ignore = None
            return

        if not key_text:
            self.completer.hide()
            super(CustomTextEdit, self).keyPressEvent(event)
            return


        if event.key() in (
            #QtCore.Qt.Key_Enter,
            QtCore.Qt.Key_Escape,
            QtCore.Qt.Key_Space,
            QtCore.Qt.Key_Control,
            QtCore.Qt.Key_Shift,
            QtCore.Qt.Key_Alt,
            QtCore.Qt.Key_Backtab,
            QtCore.Qt.Key_Up,
            QtCore.Qt.Key_Down,
            QtCore.Qt.Key_Right,
            QtCore.Qt.Key_Left,
            QtCore.Qt.Key_Backspace,
            QtCore.Qt.Key_CapsLock,
            ) or event.modifiers() in (
            QtCore.Qt.ControlModifier,
            QtCore.Qt.AltModifier,
            ):  # this is so sad!
            self.completer.hide()
            super(CustomTextEdit, self).keyPressEvent(event)
            return


        #Si el autocerrado está activado
        ttcc = self.autoInsert(event)
        if ttcc:
            self.setTextCursor(ttcc)
            return

        if event.key() == QtCore.Qt.Key_Tab:
            tc = self.textCursor()
            tc.insertText(" "*4)
            return


        if event.key() in [QtCore.Qt.Key_Enter, QtCore.Qt.Key_Enter-1]:
            tc = self.textCursor()
            pos = tc.position()
            tc.select(tc.LineUnderCursor)
            line = tc.selectedText()
            comment = ""
            if line.isspace() or line == "":
                len_s = len(line)
            else:
                normal = line.replace(" ", "")
                #if normal.startswith("//"):
                    #comment = "//"
                len_s = line.find(normal[0])

            tc.setPosition(pos)
            tc.insertText("\n"+" "*len_s+comment)
            return

        super(CustomTextEdit, self).keyPressEvent(event)

        self.show_autocomplete_if_conditions()



    #----------------------------------------------------------------------

    def smart_under_selection(self, tc):
        #word like: cdc|
        tc.movePosition(tc.WordLeft, tc.KeepAnchor)

        #word like: cdc.|
        if tc.selectedText().startswith("."): tc.movePosition(tc.WordLeft, tc.KeepAnchor)

        #word like: cdc.pri|
        tc.movePosition(tc.WordLeft, tc.KeepAnchor)
        if tc.selectedText().startswith("."): tc.movePosition(tc.WordLeft, tc.KeepAnchor)
        else: tc.movePosition(tc.WordRight, tc.KeepAnchor)


    #----------------------------------------------------------------------
    def show_autocomplete_if_conditions(self):

        tc = self.textCursor()

        if self.get_format() in ["comment", "quotation"]:
            self.completer.hide()
            return

        self.smart_under_selection(tc)
        selected = tc.selectedText().split()

        if not selected:
            self.completer.hide()
            return


        if re.match('^[\w.]+$', selected[-1]) is None:
            self.completer.hide()
            return


        if selected: self.last_w = selected[-1]

        try:
            #Si no cumple con el mínimo de letras
            if len(self.last_w) < self.completer.spell:
                self.completer.hide()

            else:
                self.completer.popup(self.getPosPopup(), self.last_w)

        except UnicodeEncodeError:
            return  #capturas tildes y caracteres especiales


    #----------------------------------------------------------------------
    def brace_match(self):

        return


    #----------------------------------------------------------------------
    def get_format(self):

        contex_color = {"#7f0000": "quotation",
                        "#cc0000": "quotation",
                        "#007f00": "comment",
                        "#c81818": "comment",}

        tc = self.textCursor()
        pos = tc.positionInBlock()

        block = tc.block()
        layout = block.layout()
        formats = layout.additionalFormats()

        for format_ in formats:
            if pos >= format_.start and pos <= format_.start + format_.length:
                return contex_color.get(format_.format.foreground().color().name().lower(), None)









#! /usr/bin/python
#-*- coding: utf-8 -*-

from PySide.QtGui import QWidget
from PySide.QtGui import QPainter
from PySide import QtCore, QtGui

#from ..methods.backgrounds import BackgroundPallete

class LineNumber(QWidget):

    #----------------------------------------------------------------------
    def __init__(self, *args, **kwargs):
        super(LineNumber, self).__init__(*args, **kwargs)

        self.edit = None
        self.highest_line = 0
        self.current = 0

        self.setMinimumSize(QtCore.QSize(51, 0))
        self.setMaximumSize(QtCore.QSize(51, 16777215))
        #BackgroundPallete.set_background_pinguino(self)

        palette = QtGui.QPalette(self.palette())
        self.setAutoFillBackground(True)
        # palette.setColor(QtGui.QPalette.Window, QtGui.QColor(175, 200, 225))
        palette.setColor(QtGui.QPalette.Window, QtGui.QColor("#E0E0E0"))
        self.setPalette(palette)

        self.setStyleSheet("""
        font-family: mono;
        font-weight: normal;
        font-size: 10pt;

        """)


    #----------------------------------------------------------------------
    def setTextEdit(self, edit):
        self.edit = edit

    #----------------------------------------------------------------------
    def update(self, *args):
        #width=50
        #if self.width() != width:
            #self.setFixedWidth(width)
        QWidget.update(self, *args)

    #----------------------------------------------------------------------
    def paintEvent(self, event):
        contents_y = self.edit.verticalScrollBar().value()
        page_bottom = contents_y + self.edit.viewport().height()
        font_metrics = self.fontMetrics()
        current_block = self.edit.document().findBlock(self.edit.textCursor().position())
        painter = QPainter(self)
        line_count = 0
        block = self.edit.document().begin()
        while block.isValid():
            line_count += 1
            position = self.edit.document().documentLayout().blockBoundingRect(block).topLeft()
            if position.y() > page_bottom:
                break
            bold = False
            if block == current_block:
                bold = True
                font = painter.font()
                font.setBold(True)
                painter.setFont(font)
                self.current = line_count
            painter.drawText(self.width() - font_metrics.width(str(line_count)) - 10,
                             round(position.y()) - contents_y + font_metrics.ascent(),
                             str(line_count))
            if bold:
                font = painter.font()
                font.setBold(False)
                painter.setFont(font)
            block = block.next()
        self.highest_line = line_count
        painter.end()

        QWidget.paintEvent(self, event)



#!/usr/bin/env python
#-*- coding: utf-8 -*-

from PySide import QtCore

from .methods.core import PinguinoCore

########################################################################
class PinguinoEvents(PinguinoCore):

    #----------------------------------------------------------------------
    def __init__(self):
        """"""
        PinguinoCore.__init__(self)


    #----------------------------------------------------------------------
    def connect_events(self):
        """"""

        # Triggers
        self.connect(self.main.actionNew_file, QtCore.SIGNAL("triggered()"), self.ide_new_file)
        self.connect(self.main.actionNew_blocks_file, QtCore.SIGNAL("triggered()"), self.PinguinoKIT.ide_new_file)
        self.connect(self.main.actionOpen_file, QtCore.SIGNAL("triggered()"), self.ide_open_files)
        self.connect(self.main.actionSave_file, QtCore.SIGNAL("triggered()"), self.ide_save_file)
        self.connect(self.main.actionSave_as, QtCore.SIGNAL("triggered()"), self.ide_save_as)
        self.connect(self.main.actionSave_all, QtCore.SIGNAL("triggered()"), self.ide_save_all)
        self.connect(self.main.actionClose_file, QtCore.SIGNAL("triggered()"), self.ide_close_file)
        self.connect(self.main.actionClose_all, QtCore.SIGNAL("triggered()"), self.ide_close_all)
        self.connect(self.main.actionClose_others, QtCore.SIGNAL("triggered()"), self.ide_close_others)
        self.connect(self.main.actionPrint, QtCore.SIGNAL("triggered()"), self.editor_print_file)
        self.connect(self.main.actionQuit, QtCore.SIGNAL("triggered()"), self.ide_close_ide)
        self.connect(self.main.actionUndo, QtCore.SIGNAL("triggered()"), self.editor_undo)
        self.connect(self.main.actionRedo, QtCore.SIGNAL("triggered()"), self.editor_redo)
        self.connect(self.main.actionCut, QtCore.SIGNAL("triggered()"), self.editor_cut)
        self.connect(self.main.actionCopy, QtCore.SIGNAL("triggered()"), self.editor_copy)
        self.connect(self.main.actionPaste, QtCore.SIGNAL("triggered()"), self.editor_paste)
        self.connect(self.main.actionDelete, QtCore.SIGNAL("triggered()"), self.editor_delete)
        self.connect(self.main.actionSelect_all, QtCore.SIGNAL("triggered()"), self.editor_select_all)
        self.connect(self.main.actionComment_out_region, QtCore.SIGNAL("triggered()"), self.editor_comment_region)
        self.connect(self.main.actionComment_Uncomment_region, QtCore.SIGNAL("triggered()"), self.editor_comment_uncomment)
        self.connect(self.main.actionIndent, QtCore.SIGNAL("triggered()"), self.editor_indent_region)
        self.connect(self.main.actionDedent, QtCore.SIGNAL("triggered()"), self.editor_dedent_region)
        # self.connect(self.main.actionHex_code, QtCore.SIGNAL("triggered()"), self.__show_hex_code__)
        self.connect(self.main.actionMain_c, QtCore.SIGNAL("triggered()"), self.ide_show_main_c)
        self.connect(self.main.actionUser_c, QtCore.SIGNAL("triggered()"), self.ide_show_user_c)
        self.connect(self.main.actionDefine_h, QtCore.SIGNAL("triggered()"), self.ide_show_define_h)
        self.connect(self.main.actionHex, QtCore.SIGNAL("triggered()"), self.ide_show_hex)
        self.connect(self.main.actionReset_IDE_instalation, QtCore.SIGNAL("triggered()"), self.ide_reset_instalation)


        # Toggle Tabs
        self.connect(self.main.actionTabLog, QtCore.SIGNAL("toggled(bool)"), lambda :self.toggle_tab("Log"))
        self.connect(self.main.actionTabFiles, QtCore.SIGNAL("toggled(bool)"), lambda :self.toggle_tab("Files"))
        self.connect(self.main.actionTabBoardConfig, QtCore.SIGNAL("toggled(bool)"), lambda :self.toggle_tab("BoardConfig"))
        self.connect(self.main.actionTabStdout, QtCore.SIGNAL("toggled(bool)"), lambda :self.toggle_tab("Stdout"))
        self.connect(self.main.actionTabSearchReplace, QtCore.SIGNAL("toggled(bool)"), lambda :self.toggle_tab("SearchReplace"))
        self.connect(self.main.actionTabProject, QtCore.SIGNAL("toggled(bool)"), lambda :self.toggle_tab("Project"))
        self.connect(self.main.actionTabShell, QtCore.SIGNAL("toggled(bool)"), lambda :self.toggle_tab("Shell"))
        self.connect(self.main.actionTabSourceBrowser, QtCore.SIGNAL("toggled(bool)"), lambda :self.toggle_tab("SourceBrowser"))
        self.connect(self.main.actionLibraryManager, QtCore.SIGNAL("toggled(bool)"), lambda :self.toggle_tab("LibraryManager"))


        # Perspective related events
        self.connect(self.main.actionMenubar, QtCore.SIGNAL("toggled(bool)"), self.toggle_menubar)
        self.connect(self.main.actionToolbars, QtCore.SIGNAL("toggled(bool)"), self.toggle_toolbars)
        self.connect(self.main.actionToggle_horizontal_tool_area, QtCore.SIGNAL("triggered()"), self.toggle_bottom_area)
        self.connect(self.main.actionToggle_vertical_tool_area, QtCore.SIGNAL("triggered()"), self.toggle_right_area)
        self.connect(self.main.tabWidget_tools, QtCore.SIGNAL("currentChanged(int)"), self.tab_right_changed)
        self.connect(self.main.tabWidget_bottom, QtCore.SIGNAL("currentChanged(int)"), self.tab_bottoms_changed)
        # self.connect(self.main.actionToggle_editor_area, QtCore.SIGNAL("toggled(bool)"), self.toggle_editor_area)
        self.connect(self.main.actionMove_vertical_tool_area, QtCore.SIGNAL("triggered()"), self.toggle_side_vertical_area)


        # Settings related events
        self.connect(self.main.actionAutocomplete, QtCore.SIGNAL("triggered()"), self.switch_autocomplete)
        self.connect(self.main.actionColor_theme, QtCore.SIGNAL("toggled(bool)"), self.switch_color_theme)
        self.connect(self.main.action16x16, QtCore.SIGNAL("triggered(bool)"), self.resize_toolbar(16, self.main.action16x16))
        self.connect(self.main.action24x24, QtCore.SIGNAL("triggered(bool)"), self.resize_toolbar(24, self.main.action24x24))
        self.connect(self.main.action32x32, QtCore.SIGNAL("triggered(bool)"), self.resize_toolbar(32, self.main.action32x32))
        self.connect(self.main.action48x48, QtCore.SIGNAL("triggered(bool)"), self.resize_toolbar(48, self.main.action48x48))


        # Child windows
        # self.connect(self.main.actionLibrary_manager, QtCore.SIGNAL("triggered()"), self.__show_libmanager__)
        self.connect(self.main.actionSet_paths, QtCore.SIGNAL("triggered()"), self.__config_paths__)
        self.connect(self.main.actionSelect_board, QtCore.SIGNAL("triggered()"), self.set_tab_board)
        # self.connect(self.main.actionView_Pinguino_code, QtCore.SIGNAL("triggered()"), self.__show_pinguino_code__)
        self.connect(self.main.actionInsert_Block, QtCore.SIGNAL("triggered()"), self.__show_insert_block__)
        self.connect(self.main.actionSubmit_bug_report, QtCore.SIGNAL("triggered()"), self.__show_submit_bug__)
        # self.connect(self.main.actionCheck_for_patches, QtCore.SIGNAL("triggered()"), self.__show_patches__)


        # Pinguinguino related events
        self.connect(self.main.actionCompile, QtCore.SIGNAL("triggered()"), self.pinguino_compile)
        self.connect(self.main.actionUpload, QtCore.SIGNAL("triggered()"), self.pinguino_upload)
        self.connect(self.main.actionUpload_hex_directly, QtCore.SIGNAL("triggered()"), self.pinguino_upload_hex)
        self.connect(self.main.actionIf_Compile_then_Upload, QtCore.SIGNAL("triggered()"), self.pinguino_compile_and_upload)


        # Help
        self.connect(self.main.actionGitPinguinoIde, QtCore.SIGNAL("triggered()"), lambda :self.open_web_site("https://github.com/PinguinoIDE/pinguino-ide/releases/latest"))
        self.connect(self.main.actionGitPinguinoLibraries, QtCore.SIGNAL("triggered()"), lambda :self.open_web_site("https://github.com/PinguinoIDE/pinguino-libraries"))
        self.connect(self.main.actionGitPinguinoCompilers, QtCore.SIGNAL("triggered()"), lambda :self.open_web_site("https://github.com/PinguinoIDE/pinguino-compilers"))
        self.connect(self.main.actionWebsite, QtCore.SIGNAL("triggered()"), lambda :self.open_web_site("http://www.pinguino.cc/"))
        self.connect(self.main.actionWiki, QtCore.SIGNAL("triggered()"), lambda :self.open_web_site("http://wiki.pinguino.cc/"))
        self.connect(self.main.actionForum, QtCore.SIGNAL("triggered()"), lambda :self.open_web_site("http://forum.pinguino.cc/"))
        self.connect(self.main.actionBlog, QtCore.SIGNAL("triggered()"), lambda :self.open_web_site("http://blog.pinguino.cc/"))
        self.connect(self.main.actionGroup, QtCore.SIGNAL("triggered()"), lambda :self.open_web_site("https://groups.google.com/forum/#!forum/pinguinocard"))
        self.connect(self.main.actionShop, QtCore.SIGNAL("triggered()"), lambda :self.open_web_site("http://shop.pinguino.cc/"))
        self.connect(self.main.actionAbout, QtCore.SIGNAL("triggered()"), self.__show_about__)
        # self.connect(self.main.actionCheck_for_updates, QtCore.SIGNAL("triggered()"), self.need_update)


        # Events
        self.closeEvent = self.ide_close_ide
        # self.connect(self.main.actionSwitch_ide, QtCore.SIGNAL("toggled(bool)"), self.switch_ide_mode)
        self.connect(self.main.tabWidget_files, QtCore.SIGNAL("currentChanged(int)"), self.ide_tab_changed)
        self.connect(self.main.tabWidget_files, QtCore.SIGNAL("tabCloseRequested(int)"), self.ide_tab_close)
        # self.connect(self.main.tabWidget_bottom, QtCore.SIGNAL("customContextMenuRequested(QPoint)"), self.ide_tabs_context_menu)
        # self.connect(self.main.tabWidget_tools, QtCore.SIGNAL("customContextMenuRequested(QPoint)"), self.ide_tabs_context_menu)
        # self.connect(self.main.tabWidget_files, QtCore.SIGNAL("tabCloseRequested(int)"), self.ide_tab_close)


        # Graphical mode
        self.connect(self.main.actionSave_image, QtCore.SIGNAL("triggered()"), self.editor_save_screen_image)
        self.connect(self.main.lineEdit_blocks_search, QtCore.SIGNAL("textChanged(QString)"), self.PinguinoKIT.update_blocks_search_tab)
        self.connect(self.main.actionExport_code_to_editor, QtCore.SIGNAL("triggered()"), self.editor_export_pinguino_code)
        self.connect(self.main.comboBox_blocks, QtCore.SIGNAL("activated(QString)"), self.set_block_tab)
        self.connect(self.main.actionGenerate_blocks, QtCore.SIGNAL("triggered()"), self.PinguinoKIT.code_to_blocks)


        # Context menu
        self.main.tabWidget_files.contextMenuEvent = self.editor_tabfile_context_menu
        self.main.tabWidget_files.contextMenuEvent = self.editor_tabfile_context_menu


        # Initialize
        self.main.actionToolbars.setChecked(True)
        # self.main.actionConfirm_board.setChecked(self.configIDE.config("Features", "confirm_board", True))

        self.main.dockWidget_right.resizeEvent = self.tab_tools_resize
        self.main.dockWidget_bottom.resizeEvent = self.tab_tools_resize

        self.main.tabWidget_bottom.contextMenuEvent = self.ide_tabs_context_menu
        self.main.tabWidget_tools.contextMenuEvent = self.ide_tabs_context_menu
        self.main.toolBar.contextMenuEvent = self.ide_tabs_context_menu
        self.main.menubar.contextMenuEvent = self.ide_tabs_context_menu



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file '/home/yeison/Documentos/Desarrollo/Pinguino/GitHub/pinguino-ide/pinguino/qtgui/frames/grafical_widget.ui'
#
# Created: Tue Nov 17 17:51:45 2015
#      by: pyside-uic 0.2.15 running on PySide 1.2.4
#
# WARNING! All changes made in this file will be lost!

from PySide import QtCore, QtGui

class Ui_Form_graphical(object):
    def setupUi(self, Form_graphical):
        Form_graphical.setObjectName("Form_graphical")
        Form_graphical.resize(450, 406)
        Form_graphical.setWindowTitle("Graphical Widget")
        self.gridLayout = QtGui.QGridLayout(Form_graphical)
        self.gridLayout.setSpacing(0)
        self.gridLayout.setContentsMargins(0, 0, 0, 0)
        self.gridLayout.setObjectName("gridLayout")
        self.scrollArea = QtGui.QScrollArea(Form_graphical)
        self.scrollArea.setFrameShape(QtGui.QFrame.NoFrame)
        self.scrollArea.setVerticalScrollBarPolicy(QtCore.Qt.ScrollBarAlwaysOn)
        self.scrollArea.setHorizontalScrollBarPolicy(QtCore.Qt.ScrollBarAlwaysOn)
        self.scrollArea.setWidgetResizable(True)
        self.scrollArea.setObjectName("scrollArea")
        self.scrollAreaWidgetContents = QtGui.QWidget()
        self.scrollAreaWidgetContents.setGeometry(QtCore.QRect(0, 0, 437, 393))
        self.scrollAreaWidgetContents.setObjectName("scrollAreaWidgetContents")
        self.gridLayout_3 = QtGui.QGridLayout(self.scrollAreaWidgetContents)
        self.gridLayout_3.setSpacing(0)
        self.gridLayout_3.setContentsMargins(0, 0, 0, 0)
        self.gridLayout_3.setObjectName("gridLayout_3")
        self.widget = QtGui.QWidget(self.scrollAreaWidgetContents)
        self.widget.setObjectName("widget")
        self.gridLayout_3.addWidget(self.widget, 0, 0, 1, 1)
        self.scrollArea.setWidget(self.scrollAreaWidgetContents)
        self.gridLayout.addWidget(self.scrollArea, 0, 0, 1, 1)

        self.retranslateUi(Form_graphical)
        QtCore.QMetaObject.connectSlotsByName(Form_graphical)

    def retranslateUi(self, Form_graphical):
        pass




#!/usr/bin/env python
#-*- coding: utf-8 -*-

import pickle
import os

from PySide.QtGui import QListWidget, QListWidgetItem
from PySide import QtCore, QtGui

from .autocomplete_icons import CompleteIcons
from ....pinguino_core.config import Config


class PinguinoAutoCompleter(QListWidget):
    """
    Rules:

    * Visible after 2 keys (self.setSpell(2))
    * Don't show with nodifiers
    * Don't show with backspace
    * When no matchs autocompleter must be visible
    * Autocompleter must hidding with space key



    """

    #----------------------------------------------------------------------
    def __init__(self):
        super(PinguinoAutoCompleter, self).__init__(None)

        self.setWindowFlags(QtCore.Qt.FramelessWindowHint |
                            QtCore.Qt.WindowSystemMenuHint |
                            QtCore.Qt.WindowStaysOnTopHint |
                            QtCore.Qt.Tool)

        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/logo/art/windowIcon.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.setWindowIcon(icon)

        self.setHorizontalScrollBarPolicy(QtCore.Qt.ScrollBarAsNeeded)
        self.setVerticalScrollBarPolicy(QtCore.Qt.ScrollBarAsNeeded)

        self.setIconSize(QtCore.QSize(14, 14))
        self.set_height(300)

        self.setSpell(2)
        self.setEnabled(True)
        self.itemsList = []
        self.itemsListName = []
        self.local_functions = {}
        self.temporal = {"variables" : [],
                         "functions" : [],
                         "directives" : [],
                         "arch8" : [],
                         "arch32" : [],
                         }

        with open(os.path.join(os.getenv("PINGUINO_USER_PATH"), "reserved.pickle"), "rb") as file_reserved:
            self.namespaces = pickle.load(file_reserved)

        self.icons = CompleteIcons()
        self.addItemsCompleter(self.namespaces["all"], self.icons.iconLibrary)
        # del icons
        self.set_arch_autocompleter()

        configIDE = Config()
        selection_color = configIDE.config("Styles", "selection_color", "#FFFFFF")
        selection_bg_color = configIDE.config("Styles", "selection_foreground_color", "#57AAFF")

        self.setStyleSheet("""
        font-family: inherit;
        font-weight: normal;
        selection-color: {};
        selection-background-color: {};
        """.format(selection_color, selection_bg_color))

        self.setFrameShape(QtGui.QFrame.NoFrame)


    #----------------------------------------------------------------------
    def set_arch_autocompleter(self):

        arch = os.getenv("PINGUINO_BOARD_ARCH")

        icons = CompleteIcons()

        if arch == "8":
            for item in self.namespaces["arch8"]:
                self.addTemporalItem("arch8", item, icons.iconLibrary)
            self.removeTemporalItems("arch32")

        elif arch == "32":
            for item in self.namespaces["arch32"]:
                self.addTemporalItem("arch32", item, icons.iconLibrary)
            self.removeTemporalItems("arch8")

        del icons

    #----------------------------------------------------------------------
    def hide(self, *args):
        super(PinguinoAutoCompleter, self).hide(*args)
        self.text_edit.setFocus()

    #----------------------------------------------------------------------
    def show(self, *args):
        #self.set_arch_autocompleter()  #FIXME: move this call
        self.activateWindow()
        super(PinguinoAutoCompleter, self).show(*args)

    #----------------------------------------------------------------------
    def focusOutEvent(self, event):
        # self.hide()
        QtGui.QListWidget.focusOutEvent(self, event)

    #----------------------------------------------------------------------
    def setEnabled(self, en):
        self.enabled = en

    #----------------------------------------------------------------------
    def set_height(self, visisble):
        self.resize(200, visisble)

    #----------------------------------------------------------------------
    def ajustWidth(self):
        hs = self.horizontalScrollBar()
        cont = 0
        while hs.isVisible() :
            self.resize(self.width()+1, self.height())
            cont += 1
            if cont > 100: return # :)

    #----------------------------------------------------------------------
    def ajustPos(self):
        screen = QtGui.QDesktopWidget().screenGeometry()
        size = self.geometry()

        cont = 0
        while self.pos().y() + size.height() - screen.height() > -10:
            self.move(self.pos().x(), self.pos().y() - 5)
            cont += 1
            if cont > 100: self.destroy()

        cont = 0
        while self.pos().x() + size.width() - screen.width() > -10:
            self.move(self.pos().x() - 5, self.pos().y())
            cont += 1
            if cont > 100: self.destroy()


    #----------------------------------------------------------------------
    def setSpell(self, int_):
        self.spell = int_

    #----------------------------------------------------------------------
    def down(self):
        current = self.currentRow()
        self.setCurrentRow(current+1)

    #----------------------------------------------------------------------
    def up(self):
        current = self.currentRow()
        self.setCurrentRow(current-1)

    #----------------------------------------------------------------------
    def addNewItem(self, name, icon=None):

        if os.getenv("PINGUINO_PYTHON") is "2":
            if not icon: icon = None
            elif type(icon) in [str, unicode]: icon = getattr(self.icons, icon)
        elif os.getenv("PINGUINO_PYTHON") is "3":
            if not icon: icon = None
            elif type(icon) == str: icon = getattr(self.icons, icon)

        if not name in self.itemsListName:
            item = QListWidgetItem()
            if icon: item.setIcon(icon)
            item.setText(name)
            self.addItem(item)
            self.itemsList.append(item)
            self.itemsListName.append(name)
            return item


    #----------------------------------------------------------------------
    def addTemporalItem(self, llave, name, icon=None):
        # if not icon: icon = None
        # elif type(icon) == str: icon = getattr(self.icons, icon)
        item = self.addNewItem(name, icon)
        if item:
            self.temporal[llave].append(item)

    #----------------------------------------------------------------------
    def update_local_functions(self, functions):
        """"""
        self.local_functions = functions


    #----------------------------------------------------------------------
    def removeTemporalItems(self, llave):

        for item in self.temporal[llave]:
            index = self.getIndex(item.text())
            if index:
                self.itemsList.pop(index)
                #self.itemsList.remove(item)
                self.itemsListName.remove(item.text())
                self.takeItem(index)
        self.update()
        self.temporal[llave] = []

    #----------------------------------------------------------------------
    def getIndex(self, name, start=False):
        cont = 0
        for item in self.itemsList:
            if start:
                if item.text().lower().startswith(name.lower()): return cont
            else:
                if item.text() == name: return cont
            cont += 1
        return None

    #----------------------------------------------------------------------
    def addItemsCompleter(self, list_, icon=None):

        if os.getenv("PINGUINO_PYTHON") is "2":
            if not icon: icon = None
            elif type(icon) in [str, unicode]: icon = getattr(self.icons, icon)
        elif os.getenv("PINGUINO_PYTHON") is "3":
            if not icon: icon = None
            elif type(icon) == str: icon = getattr(self.icons, icon)

        for text in list_:
            if not text in self.itemsListName:
                item = QListWidgetItem()
                if icon != None: item.setIcon(icon)
                item.setText(text)
                self.addItem(item)
                self.itemsList.append(item)
                self.itemsListName.append(item.text())


    #----------------------------------------------------------------------
    def popup(self, pos, index=None):

        ##hide autocompleter after 3s
        #hide_timer = QtCore.QTimer()
        #QtCore.QTimer.singleShot(3000, self.hide)
        #hide_timer.setSingleShot(True)
        #hide_timer.start()

        self.sortItems()
        self.itemsList.sort()

        if index == None:
            self.setCurrentRow(0)
            self.move(pos)
            self.show()
            self.setFocus()
            self.ajustWidth()
            self.ajustPos()
            return

        cont = self.getIndex(index, True)
        #if not cont:
            #self.hide()
            #return

        if self.enabled and cont:
            self.setCurrentRow(cont)
            item = cont + 5
            if item >= len(self.itemsList): item = -1
            self.scrollToItem(self.itemsList[item])

        self.move(pos)
        self.show()

        self.ajustWidth()
        self.ajustPos()

        self.setFocus()
        self.activateWindow()


#!/usr/bin/env python
#-*- coding: utf-8 -*-

from PySide import QtCore

Blocks = {"linear": "BlockLinear",
          "function": "BlockFunction",
          "nested": "BlockNested",
          "output": "BlockSpace",
          "output-bool": "BlockSpaceBool",
          "nested-second": "BlockNestedSecond",
          "frame-edit": "BlockFrameEdit",
          #"frame-plot": "BlockFramePlot",
          #"frame-debug": "BlockFrameDebug",
          }

CLOSEHANDCURSOR = QtCore.Qt.ClosedHandCursor
OPENHANDCURSOR = QtCore.Qt.OpenHandCursor
# CLOSEHANDCURSOR = QtCore.Qt.SizeAllCursor
# OPENHANDCURSOR = QtCore.Qt.SizeAllCursor


#!/usr/bin/env python
#-*- coding: utf-8 -*-

import os
import shutil
import argparse
import re
import codecs

from .pinguino_tools import PinguinoTools
from .boards import boardlist as AllBoards


########################################################################
class Pinguino(PinguinoTools):


    #----------------------------------------------------------------------
    def __init__(self):
        """Pinguino

        Class for handle Pinguino from Python.
        """

        super(Pinguino, self).__init__()


    #----------------------------------------------------------------------
    def compile_file(self, file_path):
        """Compile file from path.

        Parameters
        ----------
        file_path: str, list
            Full absolute or relative path of file, list of files is project.

        See Also
        --------
        compile_string: Compile from string.
        """

        if type(file_path) == str:
            file_path = [file_path]

        data = self.verify(file_path)
        self.__compiled__ = data["verified"]
        self.__data__ = data


    #----------------------------------------------------------------------
    def compile_string(self, code, tmp_filename="temp_filename.pde", filenames=[]):
        """Compile from string.

        Parameters
        ----------
        code: str
            String with Pinguino Code for compile
        tmp_filename: str, optional
            Temporal filename used for compile the code.
        filenames: list of strings, optional
            Environment files for projects.

        See Also
        --------
        compile_file: Compile from path.

        Examples
        --------
        >>> code = "setup(){pinmode(0,OUTPUT);}loop(){toggle(0);delay(100);}"
        >>> compile_string(code)
        """

        tmp_dir = os.path.join(os.getenv("PINGUINO_USER_PATH"), "temp")
        if not os.path.exists(tmp_dir):
            os.mkdir(tmp_dir)
        tmp_file = os.path.join(tmp_dir, tmp_filename)
        file_ = codecs.open(tmp_file, "w", encoding="utf-8")
        file_.write(code)
        file_.close()
        self.compile_file([tmp_file] + filenames)


    #----------------------------------------------------------------------
    def upload(self):
        """Upload previous successfully compiled file.


        Returns
        -------
         uploaded: bool
            Is True if file was uploaded succesffully, otherwise is False.

        data: list
            List of messages reports.
        """

        data = self.__upload__()
        data = "\n".join(data)
        NO_UPLOADED_KEY_TEXT = ["Pinguino not found",
                                "Error",
                                "Upload not possible",
                                "No target found",
                                "Resource busy",
                                "Entity not found",
                                ]

        if False in [data.find(key) == -1 for key in NO_UPLOADED_KEY_TEXT]:
            if data.find("Error") != -1: data = data[data.find("Error"):]
            return False, data
        return True, data


    # #----------------------------------------------------------------------
    # def set_bootloader(self, bldr, memstart):
        # """Configure bootloader for current board selected.

        # Parameters
        # ----------
        # bldr: str
            # "noboot", "boot2" or "boot4"
        # memstart: int
            # 0, 0x2000 or 0x0C00

        # See Also
        # --------
        # set_board: Set the board to use.
        # """

        # if self.__current_board__.arch == 8:
            # self.__current_board__.bldr = bldr
            # self.__current_board__.memstart = memstart

    #----------------------------------------------------------------------
    def set_icsp(self):
        """"""
        if self.__current_board__.arch == 8:
            self.__current_board__.bldr = "noboot"
            self.__current_board__.memstart = 0


    #----------------------------------------------------------------------
    def set_8bit_compiler(self, compiler):
        """"""
        assert compiler.lower() in ["sdcc", "xc8"], "Compiler must be SDCC or XC8"
        self.SET_COMPILER_8BIT = compiler.lower()


    #----------------------------------------------------------------------
    def compiled(self):
        """Check if file was successfully compiled.

        Returns
        -------
        compiled: bool
            True if file was successfully compiled, otherwise is False.
        """

        return self.__compiled__


    #----------------------------------------------------------------------
    def get_errors(self):
        """Getter for errors while compiling.

        Returns
        -------
        errors: dict
            Dictionary with 3 types o errors (keys): compiling and linking


        See Also
        --------
        get_errors_linking: Get linking errors.
        get_errors_compiling_c: Get compiling C/C++ errors.
        get_errors_compiling_asm: Get compiling assembler errors.
        """

        errors = {}
        errors["compiling"] = self.__data__["compiling"]
        errors["linking"] = self.__data__["linking"]
        return errors


    #----------------------------------------------------------------------
    def get_errors_linking(self):
        """Getter for linking errors.

        Returns
        -------
        error: dict, None
            Dictionary with error message or None.

        See Also
        --------
        get_errors: Get errors while compiling.
        get_errors_compiling_c: Get compiling C/C++ errors.
        get_errors_compiling_asm: Get compiling assembler errors.
        """

        errors = self.__data__["linking"]
        if not errors: return None

        error = {}
        error["linking"] = errors
        error["line_numbers"] = []
        for err in errors:
            match = re.match(".*user.c:([\d]+):[\d]+: error.*", err)
            if not match is None: error["line_numbers"] += [int(match.groups()[0])]
        return error


    #----------------------------------------------------------------------
    def get_errors_compiling_c(self):
        """Getter for compiling C/C++ errors.

        Returns
        -------
        error: dict
            Dictionary with error message or None.

        See Also
        --------
        get_errors: Get errors while compiling.
        get_errors_linking: Get linking errors.
        get_errors_compiling_asm: Get compiling assembler errors.
        """

        errors = self.__data__["compiling"]["c"]
        if not errors: return None

        error = {}
        error["line_numbers"] = []
        error["complete_message"] = ""
        error["messages"] = {}
        for err in errors:
            error["line_numbers"] += [int(err["line_number"])]
            error["complete_message"] += err["line_number"] + ": " + err["message"]
            error["messages"][err["line_number"]] = err["message"]
        return error


    #----------------------------------------------------------------------
    def get_errors_compiling_asm(self):
        """Getter for compiling assembler errors.

        Returns
        -------
        error: str, None
            String with error message or None.

        See Also
        --------
        get_errors: Get errors while compiling.
        get_errors_linking: Get linking errors.
        get_errors_compiling_asm: Get compiling assembler errors.
        """

        errors = self.__data__["compiling"]["asm"]
        if not errors: return None

        if (type(errors) is dict) and ("error_symbols" in errors):
            return errors["error_symbols"]
        else:
            return errors


    #----------------------------------------------------------------------
    def get_result(self):
        """Getter log for post-compiling process.

        Returns
        -------
        result: dict
            Dictionary with info (keys) about: time, hex_file, code_size
        """

        assert self.__compiled__, "Must compile first."
        result = {}
        result["time"] = self.__data__["time"]
        result["hex_file"] = self.__data__["hex_file"]
        result["code_size"] = self.__data__["code_size"]
        return result


    #----------------------------------------------------------------------
    def get_hex(self):
        """Getter for hex file.

        Returns
        -------
        file: file
            Readonly hex file object.
        """

        assert self.__compiled__, "Must compile first."
        return open(self.__data__["hex_file"], "r")




#! /usr/bin/python
#-*- coding: utf-8 -*-

from PySide import QtGui, QtCore

MESSAGES = [
    ("CRITICAL", "#FF0000"),
    ("DEBUG", "#00FF00"),
    ("ERROR", "#FF0000"),
    ("INFO", "#1E90FF"),
    ("WARNING", "#FFA500"),
]

exceptions = [
    "GeneratorExit",
    "ImportError",
    "ReferenceError",
    "RuntimeError",
    "UnicodeTranslateError",
    "KeyError",
    "StopIteration",
    "PendingDeprecationWarning",
    "EnvironmentError",
    "LookupError",
    "BufferError",
    "OSError",
    "DeprecationWarning",
    "UnicodeError",
    "UnicodeEncodeError",
    "FloatingPointError",
    "UnicodeWarning",
    "NameError",
    "BytesWarning",
    "IOError",
    "SyntaxError",
    "FutureWarning",
    "ImportWarning",
    "SystemExit",
    "Exception",
    "EOFError",
    "StandardError",
    "ValueError",
    "TabError",
    "ZeroDivisionError",
    "SystemError",
    "IndentationError",
    "AssertionError",
    "UnicodeDecodeError",
    "TypeError",
    "IndexError",
    "RuntimeWarning",
    "KeyboardInterrupt",
    "UserWarning",
    "SyntaxWarning",
    "MemoryError",
    "UnboundLocalError",
    "ArithmeticError",
    "Warning",
    "NotImplementedError",
    "AttributeError",
    "OverflowError",
    "BaseException",
]

EXCEPTIONS = [(f,"#FF0000") for f in exceptions]
RESERVED_PYTHON = "def|class|for|while|pass|try|except|if|else|elif"


########################################################################
class Highlighter(QtGui.QSyntaxHighlighter):

    #----------------------------------------------------------------------
    def __init__(self, parent, extra=[], python=True):
        super(Highlighter, self).__init__(parent)
        color = QtGui.QColor

        self.highlightingRules = []

        if python:
            operators = QtGui.QTextCharFormat()
            operators.setFontWeight(QtGui.QFont.Bold)
            self.highlightingRules.append(("[()\[\]{}<>=\-\+\*\\%#!~&^,/]", operators))

            reserved = QtGui.QTextCharFormat()
            reserved.setForeground(color("#8ae234"))
            #self.highlightingRules.append(("\\b(None|False|True|def|class|for|while|pass|try|except|print|if)\\b", reserved))
            self.highlightingRules.append(("\\b({})\\b".format(RESERVED_PYTHON), reserved))

        if extra:
            start_command = QtGui.QTextCharFormat()
            start_command.setForeground(color("#729fcf"))
            self.highlightingRules.append((extra[0].replace(".", "\."), start_command))

        sdcc_error_01 = QtGui.QTextCharFormat()
        sdcc_error_01.setForeground(color("#ef292a"))
        self.highlightingRules.append(("ERROR: .*", sdcc_error_01))

        for msg, color_ in MESSAGES:
            debugger = QtGui.QTextCharFormat()
            debugger.setForeground(color(color_))
            self.highlightingRules.append(("\[{}\] .*".format(msg), debugger))

        for msg, color_ in EXCEPTIONS:
            debugger = QtGui.QTextCharFormat()
            debugger.setForeground(color(color_))
            self.highlightingRules.append(("\\b({})\\b".format(msg), debugger))


    #----------------------------------------------------------------------
    def highlightBlock(self, text):
        for pattern, format_ in self.highlightingRules:
            expression = QtCore.QRegExp(pattern)
            index = expression.indexIn(text)
            while index >= 0:
                length = expression.matchedLength()
                self.setFormat(index, length, format_)
                index = expression.indexIn(text, index + length)



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file '/home/yeison/Documentos/Desarrollo/Pinguino/GitHub/pinguino-ide/pinguino/qtgui/gide/bloques/widgets/label.ui'
#
# Created: Tue Nov 17 17:51:41 2015
#      by: pyside-uic 0.2.15 running on PySide 1.2.4
#
# WARNING! All changes made in this file will be lost!

from PySide import QtCore, QtGui

class Ui_Frame(object):
    def setupUi(self, Frame):
        Frame.setObjectName("Frame")
        Frame.resize(94, 23)
        Frame.setWindowTitle("")
        self.gridLayout = QtGui.QGridLayout(Frame)
        self.gridLayout.setSpacing(0)
        self.gridLayout.setContentsMargins(0, 0, 0, 0)
        self.gridLayout.setObjectName("gridLayout")
        self.label = QtGui.QLabel(Frame)
        font = QtGui.QFont()
        font.setFamily("Ubuntu Mono")
        font.setPointSize(15)
        font.setWeight(75)
        font.setBold(True)
        self.label.setFont(font)
        self.label.setStyleSheet("color: rgb(255, 255, 255);")
        self.label.setText("TextLabel")
        self.label.setObjectName("label")
        self.gridLayout.addWidget(self.label, 0, 0, 1, 1)

        self.retranslateUi(Frame)
        QtCore.QMetaObject.connectSlotsByName(Frame)

    def retranslateUi(self, Frame):
        pass




#!/usr/bin/env python
#-*- coding: utf-8 -*-

from . import constructor as Block
from .constructor import Kit

Kit = Kit()

########################################################################
class Math_(object):

    sin = lambda n:Kit.addOper("Math", "sin", "output", ["space"])
    cos = lambda n:Kit.addOper("Math", "cos", "output", ["space"])
    tan = lambda n:Kit.addOper("Math", "tan", "output", ["space"])
    cot = lambda n:Kit.addOper("Math", "cot", "output", ["space"])
    sec = lambda n:Kit.addOper("Math", "sec", "output", ["space"])
    csc = lambda n:Kit.addOper("Math", "csc", "output", ["space"])

    add = lambda n:Kit.addOper("Math", "+", "output", ["space", "space"])
    sub = lambda n:Kit.addOper("Math", "-", "output", ["space", "space"])
    pro = lambda n:Kit.addOper("Math", "*", "output", ["space", "space"])
    div = lambda n:Kit.addOper("Math", "/", "output", ["space", "space"])
    mod = lambda n:Kit.addOper("Math", "%", "output", ["space", "space"])

    may = lambda n:Kit.addOper("Math", ">", "output-bool", ["space", "space"])
    mayeq = lambda n:Kit.addOper("Math", ">=", "output-bool", ["space", "space"])
    men = lambda n:Kit.addOper("Math", "<", "output-bool", ["space", "space"])
    meneq = lambda n:Kit.addOper("Math", "<=", "output-bool", ["space", "space"])

    equ = lambda n:Kit.addOper("Math", "==", "output-bool", ["space", "space"])
    asig = lambda n:Kit.addOper("Math", "=", "output", ["space", "space"])
    nequ = lambda n:Kit.addOper("Math", "!=", "output-bool", ["space", "space"])
    equ_b = lambda n:Kit.addOper("Math", "==", "output-bool", ["space_bool", "space_bool"])
    nequ_b = lambda n:Kit.addOper("Math", "!=", "output-bool", ["space_bool", "space_bool"])


    #----------------------------------------------------------------------
    def map_(self):

        pm = Block.Output("Math")
        pm.addLabel("map")
        pm.addSyntax("(")
        pm.addSpace()
        pm.addSyntax(",")
        pm.addDecorator(" ")
        pm.addSpace()
        pm.addSyntax(",")
        pm.addDecorator("~")
        pm.addSpace()
        pm.addSyntax(",")
        pm.addDecorator(" ")
        pm.addSpace()
        pm.addSyntax(",")
        pm.addDecorator("~")
        pm.addSpace()
        pm.addSyntax(")")
        return pm.getBlock()


    #----------------------------------------------------------------------
    def agrup(self):

        pm = Block.Output("Math")
        pm.addLabel("(")
        pm.addSpace()
        pm.addLabel(")")
        return pm.getBlock()


    #----------------------------------------------------------------------
    def agrup_bool(self):

        pm = Block.Output("Math")
        pm.addLabel("(")
        pm.addSpaceBool()
        pm.addLabel(")")
        return pm.getBlock()


    #----------------------------------------------------------------------
    def incre__(self):

        pm = Block.Output("Math")
        pm.addSpace()
        pm.addLabel("++")
        return pm.getBlock()


    #----------------------------------------------------------------------
    def decre__(self):

        pm = Block.Output("Math")
        pm.addSpace()
        pm.addLabel("--")
        return pm.getBlock()


    #----------------------------------------------------------------------
    def pre_incre(self):

        pm = Block.Output("Math")
        pm.addLabel("++")
        pm.addSpace()
        return pm.getBlock()


    #----------------------------------------------------------------------
    def pre_decre(self):

        pm = Block.Output("Math")
        pm.addLabel("--")
        pm.addSpace()
        return pm.getBlock()

    ##  Logical operators

    #----------------------------------------------------------------------
    def and_(self):

        pm = Block.OutputBool("Math")
        pm.addSpace()
        pm.addDecorator("AND")
        pm.addSyntax("&&")
        pm.addSpace()
        pm.addContextHelp("Logical AND")
        return pm.getBlock()


    #----------------------------------------------------------------------
    def or_(self):

        pm = Block.OutputBool("Math")
        pm.addSpace()
        pm.addDecorator("OR")
        pm.addSyntax("||")
        pm.addSpace()
        pm.addContextHelp("Logical OR")
        return pm.getBlock()


    #----------------------------------------------------------------------
    def not_o(self):

        pm = Block.OutputBool("Math")
        pm.addDecorator("NOT")
        pm.addSyntax("!")
        pm.addSpace()
        pm.addContextHelp("Logical NOT")
        return pm.getBlock()


    #----------------------------------------------------------------------
    def not_b(self):

        pm = Block.OutputBool("Math")
        pm.addDecorator("NOT")
        pm.addSyntax("!")
        pm.addSpaceBool()
        pm.addContextHelp("Logical NOT")
        return pm.getBlock()


    #----------------------------------------------------------------------
    def xor_(self):

        pm = Block.OutputBool("Math")
        pm.addSyntax("(!")
        pm.addSpaceBool()
        pm.addDecorator("XOR")
        pm.addSyntax(") != (!")
        pm.addSpaceBool()
        pm.addSyntax(")")
        pm.addContextHelp("Logical XOR")
        return pm.getBlock()


    #----------------------------------------------------------------------
    def xnor_(self):

        pm = Block.OutputBool("Math")
        pm.addSyntax("(!")
        pm.addSpaceBool()
        pm.addDecorator("XNOR")
        pm.addSyntax(") == (!")
        pm.addSpaceBool()
        pm.addSyntax(")")
        pm.addContextHelp("Logical XNOR")
        return pm.getBlock()



#!/usr/bin/env python
#-*- coding: utf-8 -*-

import os

from ..custom_widgets import PinguinoTextEdit


########################################################################
class PythonShell(object):

    def __init__(self):

        self.main.plainTextEdit_output = PinguinoTextEdit()
        self.main.plainTextEdit_output.set_extra_args(**{"pinguino": self})
        self.main.gridLayout_shell.addWidget(self.main.plainTextEdit_output, 0, 0, 1, 1)

# import inspect
# import types
# from ..methods.decorators import Decorator
# for name, fn in inspect.getmembers(PinguinoTextEdit):
    # if isinstance(fn, types.UnboundMethodType):
        # setattr(PinguinoTextEdit, name, Decorator.debug_method()(fn))


#!/usr/bin/env python
#-*- coding: utf-8 -*-

from PySide import QtGui

########################################################################
class BackgroundPallete(object):


    #----------------------------------------------------------------------
    def __init__(self):
        """Constructor"""

        self.PinguinoColor = "#AFC8E1"  #RGB(175, 200, 225)


    #----------------------------------------------------------------------
    def set_background_pinguino(self, widget):

        palette = QtGui.QPalette(widget.palette())
        widget.setAutoFillBackground(True)
        palette.setColor(QtGui.QPalette.Window, QtGui.QColor(self.PinguinoColor))
        widget.setPalette(palette)


    ##----------------------------------------------------------------------
    #def set_active_background(self, widget):
        #
        #palette = QtGui.QPalette(widget.palette())
        #widget.setAutoFillBackground(True)
        #palette.setColor(QtGui.QPalette.Active, QtGui.QColor(self.PinguinoColor))
        #widget.setPalette(palette)



    #----------------------------------------------------------------------
    def save_palette(self, widget):

        self.default_palette = QtGui.QPalette(widget.palette())


    #----------------------------------------------------------------------
    def get_default_palette(self):

        return self.default_palette


    #----------------------------------------------------------------------
    def set_default_palette(self, widget):

        widget.setAutoFillBackground(True)
        widget.setPalette(self.get_default_palette())


    #----------------------------------------------------------------------
    def set_palette(self, widget, palette):

        widget.setAutoFillBackground(True)
        widget.setPalette(palette)

